<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 153 $)
*/
namespace extension\ka_extensions;

class ControllerKaMultivendor extends ControllerSettings {

	protected $ext_code = 'ka_multivendor';
	
	protected $extension_version = '1.0.0.1';
	protected $min_store_version = '3.0.0.0';
	protected $max_store_version = '3.0.9.9';
	protected $min_ka_extensions_version = '4.1.1.0';
	protected $max_ka_extensions_version = '4.1.1.50';
	
	private $extension_pages = array(
		'vendor',
	);
	
	protected function onLoad() {

		$this->tables = array(		
			'ka_vendor' => array(
				'query' => "
					CREATE TABLE `" . DB_PREFIX . "ka_vendor` (
						  `customer_id` int(11) NOT NULL,
						  `vendor_custom_field` text NOT NULL,
						  `vendor_status` tinyint(1) NOT NULL,
						  `product_limit` int(10) NOT NULL,
						  `store_commission` decimal(6,2) NOT NULL,
						  `vendor_logo` varchar(255) NOT NULL,
						  UNIQUE KEY `customer_id` (`customer_id`)
						) DEFAULT CHARSET=utf8
				",
 				'is_new' => true,
 				'fields' => array( 				
  					'customer_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'vendor_custom_field' => array(
  						'type' => 'text',
  					), 				
  					'vendor_status' => array(
  						'type' => 'tinyint(1)',
  					), 				
  					'product_limit' => array(
  						'type' => 'int(10)',
  					), 				
  					'store_commission' => array(
  						'type' => 'decimal(6,2)',
  					), 				
  					'vendor_logo' => array(
  						'type' => 'varchar(255)',
  					),  					
				),
				'indexes' => array(
					'customer_id' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor` ADD INDEX (`customer_id`)",
					),
				),
			),

			'ka_vendor_order' => array(
				'query' => "
					CREATE TABLE `" . DB_PREFIX . "ka_vendor_order` (
						  `vendor_order_id` int(11) NOT NULL AUTO_INCREMENT,
						  `vendor_id` int(11) NOT NULL,
						  `main_order_id` int(11) NOT NULL,
						  `delivery_status_id` int(11) NOT NULL,
						  `shipping_method` varchar(128) NOT NULL,
						  `shipping_code` varchar(128) NOT NULL,
						  `total` decimal(15,4) NOT NULL,
						  `shipping_cost` decimal(15,4) NOT NULL,
						  `store_commission` decimal(6,2) NOT NULL,
						  `commission_total` decimal(15,4) NOT NULL,
						  PRIMARY KEY (`vendor_order_id`),
						  KEY `vendor_id` (`vendor_id`),
						  KEY `main_order_id` (`main_order_id`)
						) DEFAULT CHARSET=utf8
				",
 				'is_new' => true,
 				'fields' => array( 				
  					'vendor_order_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'vendor_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'main_order_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'delivery_status_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'shipping_method' => array(
  						'type' => 'varchar(128)',
  					), 				
  					'shipping_code' => array(
  						'type' => 'varchar(128)',
  					), 				
  					'total' => array(
  						'type' => 'decimal(15,4)',
  					), 				
  					'shipping_cost' => array(
  						'type' => 'decimal(15,4)',
  					), 				
  					'store_commission' => array(
  						'type' => 'decimal(6,2)',
  					), 				
  					'commission_total' => array(
  						'type' => 'decimal(15,4)',
  					),  					
				),
				'indexes' => array(
					'PRIMARY' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor_order` ADD PRIMARY KEY (`vendor_order_id`)",
							),
					'vendor_id' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor_order` ADD INDEX (`vendor_id`)",
					),
					'main_order_id' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor_order` ADD INDEX (`main_order_id`)",
					),
				),
			),
			'ka_vendor_description' => array(
				'query' => "
					CREATE TABLE `" . DB_PREFIX . "ka_vendor_description` (
						  `customer_id` int(11) NOT NULL,
						  `language_id` int(11) NOT NULL,
						  `name` varchar(255) NOT NULL,
						  `description` text NOT NULL,
						  PRIMARY KEY (`customer_id`,`language_id`),
						  KEY `name` (`name`)
						) CHARSET=utf8
				",
 				'is_new' => true,
 				'fields' => array( 				
  					'customer_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'language_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'name' => array(
  						'type' => 'varchar(255)',
  					), 				
  					'description' => array(
  						'type' => 'text',
  					),  					
				),
				'indexes' => array(
					'PRIMARY' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor_description` ADD PRIMARY KEY (`customer_id`,`language_id`)",
							),
					'name' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_vendor_description` ADD INDEX (`name`)",
					),
				),
			),
			'ka_category_attribute_groups' => array(
				'query' => "
					CREATE TABLE `" . DB_PREFIX . "ka_category_attribute_groups` (
						  `category_id` int(11) NOT NULL,
						  `attribute_group_id` int(11) NOT NULL,
						  PRIMARY KEY (`category_id`,`attribute_group_id`)
						) DEFAULT CHARSET=utf8
				",
 				'is_new' => true,
 				'fields' => array( 				
  					'category_id' => array(
  						'type' => 'int(11)',
  					), 				
  					'attribute_group_id' => array(
  						'type' => 'int(11)',
  					),  					
				),
				'indexes' => array(
					'PRIMARY' => array(
						'query' => "ALTER TABLE `" . DB_PREFIX . "ka_category_attribute_groups` ADD PRIMARY KEY (`category_id`,`attribute_group_id`)",
							),
				),
			),			
			'order_history' => array(
				'fields' => array(
					'vendor_order_id' => array(
						'type' => 'int(11)',
						'query' => "ALTER TABLE `" . DB_PREFIX . "order_history` ADD `vendor_order_id` INT(11) NOT NULL",
					)
				)
			),
			'order_product' => array(
				'fields' => array(
					'vendor_id' => array(
						'type' => 'int(11)',
						'query' => "ALTER TABLE `" . DB_PREFIX . "order_product` ADD `vendor_id` INT(11) NOT NULL",
					)
				)
			),
			'product' => array(
				'fields' => array(
					'vendor_id' => array(
						'type' => 'int(11)',
						'query' => "ALTER TABLE `" . DB_PREFIX . "product` ADD `vendor_id` INT(11) NOT NULL",
					)
				)
			),
		);
		
		return parent::onLoad();
	}

	
	protected function getFields() {
	
		$fields = array(
			'ka_multivendor_default_products_limit' => array(
				'code' => 'ka_multivendor_default_products_limit',
				'default_value' => 50,
				'required' => true,
				'type' => 'number',
			),
			'ka_multivendor_default_commision' => array(
				'code' => 'ka_multivendor_default_commision',
				'default_value' => 10,
				'type' => 'number',
			),
			'ka_multivendor_confirm_status' => array(
				'code' => 'ka_multivendor_confirm_status',
				'default_value' => 0,
				'type' => 'select',
			),
			'ka_multivendor_ask_delivery_confirmation' => array(
				'code' => 'ka_multivendor_ask_delivery_confirmation',
				'default_value' => true,
				'type' => 'checkbox',
			),
			'ka_multivendor_vendor_statuses' => array(
				'code' => 'ka_multivendor_vendor_statuses',
				'default_value' => array(),
				'options' => array(),
				'type' => 'checkboxes',
			),
			'ka_multivendor_shipping_setup' => array(
				'code' => 'ka_multivendor_shipping_setup',
				'default_value' => 'global',
				'type' => 'radio',
				'options' => array('global', 'custom'),
			),
		);
		
		$this->load->model('localisation/order_status');
		$order_statuses = $this->model_localisation_order_status->getOrderStatuses();
		
		$this->data['order_statuses'] = array();
		foreach ($order_statuses as $o) {
			$this->data['order_statuses'][$o['order_status_id']] = $o['name'];
		}
		$fields['ka_multivendor_vendor_statuses']['options'] = $this->data['order_statuses'];
		$fields['ka_multivendor_confirm_status']['options'] = $this->data['order_statuses'];
		
		return $fields;
	}
	
	
	public function index() {

		$this->disableRender();
		parent::index();
		$this->enableRender();

		$this->data['is_ka_smethods_installed'] = \KaGlobal::isKaInstalled('ka_smethods');
		
		$this->showPage('extension/ka_extensions/ka_multivendor/settings');
	}
	
	protected function getExtensionPages() {
		return $this->extension_pages;
	}
}

class_alias(__NAMESPACE__ . '\ControllerKaMultivendor', 'ControllerExtensionKaExtensionsKaMultivendor');