<?php
namespace extension\ka_extensions\ka_multivendor;

class ControllerCustomerCustomer extends \ControllerCustomerCustomer {
	use \extension\ka_extensions\TraitController;
	use \extension\ka_extensions\ka_multivendor\TraitControllerVendor;

	function __construct($registry) {
		parent::__construct($registry);
		$this->load->language('extension/ka_extensions/ka_multivendor/account');
	}
	
	protected function getForm() {
	
		$this->load->disableRender();
		parent::getForm();
		$this->load->enableRender();
		
		$data = $this->getRenderData();
		$template = $this->getRenderTemplate();

		$data['image_placeholder'] = $this->model_tool_image->resize('no_image.png', 100, 100);
				
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		$vendor_data = $this->getVendorData();
		
		$data = array_merge($data, $vendor_data);

		if (isset($this->request->post['vendor_status'])) {
			$data['vendor_status'] = $this->request->post['vendor_status'];
		} elseif (!empty($data['customer_id'])) {
			$data['vendor_status'] = $kamodel_vendor->getVendorStatus($data['customer_id']);
		} else {
			$data['vendor_status'] = 0;
		}

		if (empty($vendor_data['is_vendor_registered'])) {
			// checkox for vendor registration request 
			// it shows only when the vendor account is not registered for the customer
			//
			if (isset($this->request->post['register_vendor'])) {
				$data['register_vendor']    = true;
			} else {
				$data['register_vendor'] = false;
			}
			
			$data['is_vendor_registered'] = false;
		} else {
			$data['is_vendor_registered'] = true;
		}
		
		$data['is_ka_multivendor_installed'] = true;
		
		$this->response->setOutput($this->load->view($template, $data));
	}

	
	/*
		Returns an array of fields to ouput them on a customer edit page.
		
	*/
	protected function getVendorData() {
	
		$data = array();

		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
	
		$customer_id = 0;		
		if (!empty($this->request->get['customer_id'])) {
			$customer_id = (int)$this->request->get['customer_id'];
		}
	
		$vendor_status = $kamodel_vendor->getVendorStatus($customer_id);
		
		$vendor = array();
		if (!empty($customer_id)) {
			$vendor = $kamodel_vendor->getVendor($customer_id);
		}

		$fields = $kamodel_vendor->getVendorFields();

		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$data['vendor_fields'] = $this->getFieldsWithData($fields, $vendor, $this->request->post, $this->error);
		} else {
			$data['vendor_fields'] = $this->getFieldsWithData($fields, $vendor);
		}
		
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();
		$data['languages'] = $languages;
		
		// vendor description
		//
		$vendor_description = $kamodel_vendor->getVendorDescription($customer_id);
		$fields = $kamodel_vendor->getVendorDescriptionFields();

		$data['vd_fields'] = array();
		foreach ($languages as $l) {
			
			$values = (!empty($vendor_description[$l['language_id']]) ? $vendor_description[$l['language_id']] : array());
			
			if ($this->request->server['REQUEST_METHOD'] == 'POST') {
				$post   = (!empty($this->request->post['vendor_description'][$l['language_id']]) ? $this->request->post['vendor_description'][$l['language_id']] : null);
				$errors = (!empty($this->errors['vendor_description'][$l['language_id']]) ? $this->errors['vendor_description'][$l['language_id']] : array());
				$data['vd_fields'][$l['language_id']] = $this->getFieldsWithData($fields, $values, $post, $errors);
			} else {
				$data['vd_fields'][$l['language_id']] = $this->getFieldsWithData($fields, $values);
			}
		}		
		
		if (!empty($vendor)) {
			$data['is_vendor_registered'] = true;
			$data['is_awaiting_approval'] = $kamodel_vendor->isAwaitingApproval($customer_id);
			$data['customer_approvals_link'] = $this->url->link('customer/customer_approval', '&user_token=' . $this->session->data['user_token'], true);
		}
		
		return $data;
	}
	
	/*
		Validate the post data from to vendor edit page
	*/
	protected function validateForm() {

		if (!parent::validateForm()) {
			return false;
		}
	
		if (empty($this->request->post['register_vendor'])
		&& empty($this->request->post['update_vendor'])
		) {
			return true;
		}
	
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
	
		$fields = $kamodel_vendor->getVendorFields();
	
		// Custom field validation
		$this->load->model('customer/custom_field');

		// Customer Group
		//
		$customer_group_id = $this->config->get('config_customer_group_id');
		
		$custom_fields = $kamodel_vendor->getCustomFields($customer_group_id);
		
		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] != 'v_acc') {
				continue;
			}
			
			if (!empty($custom_field['required']) && empty($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']])) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			} elseif (($custom_field['type'] == 'text') 
				&& !empty($custom_field['validation']) 
				&& !filter_var($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']], 
					FILTER_VALIDATE_REGEXP, 
					array('options' => array('regexp' => $custom_field['validation']))
				)
			) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		
		
		// check vendor description fields
		//
		$fields = $kamodel_vendor->getVendorDescriptionFields();
		
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();
		
		$lang_errors = array();
		foreach ($languages as $l) {
		
			$post = array();
			if (!empty($this->request->post['vendor_description'])
				&& !empty($this->request->post['vendor_description'][$l['language_id']])
			) {
				$post = $this->request->post['vendor_description'][$l['language_id']];
			}
			
			$errors = array();
			foreach ($fields as $k => $f) {
				if (!empty($f['required'])) {
					if (empty($post[$k])) {
						$errors[$k] = str_replace('%field%', $k, $this->language->get('Field "%field%" cannot be empty'));
					}
				}
			}
			if (!empty($errors)) {
				$lang_errors[$l['language_id']] = $errors;
			}
		}
		if (!empty($lang_errors)) {
			$this->errors['vendor_description'] = $lang_errors;
		}
		
		return !$this->errors;
	}
	
}