<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 143 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerSaleOrder extends \ControllerSaleOrder {

	use \extension\ka_extensions\TraitController;

	protected function getVendorOrders($order_id, $data) {

		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		$kamodel_order_status = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order_status');
		
		$this->load->model('sale/order');
		$order_info = $this->model_sale_order->getOrder($order_id);
		
		// arrange products by vendor
		$products = array();
		foreach ($data['products'] as $p) {
			$products[$p['vendor_id']][] = $p;
		}
		
		// get vendor orders and fill them in with necessary information
		//
		$orders = $kamodel_order->getVendorOrdersByMainOrder($order_id);
		
		$vendor_orders = array();
		foreach ($orders as $vo) {

			if (!empty($products[$vo['vendor_id']])) {			
				$vo['products'] = $products[$vo['vendor_id']];
			} else {
				$vo['products'] = array();
			}
			
			$vo['vendor_info'] = $kamodel_vendor->getVendor($vo['vendor_id']);
			$shipping_cost = $this->currency->format($vo['shipping_cost'], $order_info['currency_code'], $order_info['currency_value']);
			
			$vo['commission_total'] = $this->currency->format($vo['commission_total'], $order_info['currency_code'], $order_info['currency_value']);

			$totals = array();
			
			// add the shipping cost to totals
			//
			if (!empty($vo['shipping_code'])) {
				$total = array(
					'title' => str_replace('%shipping%', $vo['shipping_method'], $this->language->get('Shipping (%shipping%)')), 
					'text'  => $shipping_cost,
				);
				$totals['shipping'] = $total;
			}
			
			$vo['totals'] = $totals;
				
			
			$vendor_orders[$vo['vendor_id']] = $vo;			
		}
		
		return $vendor_orders;
	}
	
	
	/*
		Ajax Call
		
		Returns a section with the vendor order status and other vendor-related fields
		
		It is placed on the order info page
		
		Get Parameters:
			vendor_order_id - 
		
	*/
	public function vendorOrderFields() {
		
		$this->load->language('sale/order');

		if (empty($this->request->get['vendor_order_id'])) {
			die('vendor_order_id was not found');
		}

		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		$kamodel_order_status = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order_status');
		
		$vorder = $kamodel_order->getVendorOrder($this->request->get['vendor_order_id']);
		if (empty($vorder)) {
			die('vorder was not found');
		}
		
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		$vendor = $kamodel_vendor->getVendor($vorder['vendor_id']);
		if (empty($vendor)) {
			$vendor = array();
		}
		
		$vorder['vendor_info'] = $vendor;

		$vorder['commission_total'] = $this->currency->format($vorder['commission_total'], $vorder['currency_code'], $vorder['currency_value']);
		
		$data = array();
		$data['vorder'] = $vorder;
		$data['delivery_statuses'] = $kamodel_order_status->getOrderStatusesWithVendorMark();
	
		$this->response->setOutput($this->load->view('extension/ka_extensions/ka_multivendor/admin/order_vendor', $data));
	}
	
	
	public function info() {
	
		$this->load->disableRender();
		parent::info();
		$this->load->enableRender();

		$data     = $this->getRenderData();
		$template = $this->getRenderTemplate();
		
		if (empty($data['order_id'])) {
			return;
		}
		
		if ($template != 'sale/order_info') {
			$this->log->write("The template ($template) does not match the expected template (sale/order_info)");
			return;
		}
	
		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		$kamodel_order_status = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order_status');
		
		if (!empty($kamodel_order) && $kamodel_order->isMultivendorOrder($this->request->get['order_id'])) {
			$data['vendor_orders']     = $this->getVendorOrders($this->request->get['order_id'], $data);
			$data['delivery_statuses'] = $kamodel_order_status->getOrderStatusesWithVendorMark();			
		}

		$this->response->setOutput($this->load->view('sale/order_info', $data));
	}
}