<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 146 $)
*/

namespace extension\ka_extensions\ka_multivendor;

use extension\ka_extensions\QB;

class ModelVendor extends AModelVendor {

	private $kamodel_product;

	public function deleteVendor($customer_id) {
	
		// delete vendor products
		//
		$this->load->model('catalog/product');
		$this->kamodel_product = $this->load->kamodel('extension/ka_extensions/ka_multivendor/product');
		
		$products = $this->kamodel_product->getProductsByVendorId($customer_id);
		
		if (empty($products)) {
			return;
		}
		
		foreach ($products as $p) {
			$this->model_catalog_product->deleteProduct($p['product_id']);
		}
		
		// delete vendor profile
		//
		$this->db->query("DELETE FROM " . DB_PREFIX . "ka_vendor
			WHERE customer_id = '" . (int)$customer_id . "'
		");
	}
	

	public function getVendor($customer_id) {
	
		$vendor = $this->getVendorInternal($customer_id);
		if (empty($vendor)) {
			return $vendor;
		}
		
		if (empty($vendor['name'])) {
			$vendor['name'] = $vendor['firstname'] . ' ' . $vendor['lastname'];
		}		
		
		$this->load->model('customer/customer');
		$customer = $this->model_customer_customer->getCustomer($customer_id);
		
		if (!empty($customer)) {
			$vendor['href'] = $this->url->link('customer/customer/edit', 'user_token=' . $this->session->data['user_token'] . '&customer_id=' . $vendor['customer_id'], true);
		}

		return $vendor;
	}

	
	protected function getVendorsQB($filter) {
	
		$qb = new QB();
		
		$qb->select("*", "customer", "c");
		$qb->innerJoin("ka_vendor", "ka_v", "c.customer_id = ka_v.customer_id");
		$qb->innerJoin("ka_vendor_description", "ka_vd", "c.customer_id = ka_vd.customer_id");
		
		if (!empty($filter['filter_name'])) {
			$qb->where('c.firstname = %' . $this->db->escape($filter['filter_name']) . '%');
		}
		
		if (isset($filter['start']) && isset($filter['limit'])) {
			$qb->limit($filter['start'], $filter['limit']);
		}

		$qb->orderBy('c.lastname ASC');
		
		return $qb;
	}
	
	
	public function getVendors($filter = array()) {

		$qb = $this->getVendorsQB($filter);
		
		$vendors = $qb->query()->rows;
		
		return $vendors;
	}
	
	
	public function getVendorFields() {
	
		$fields = array();
	
		$vendor_fields = array(
			'product_limit' => array(
				'code' => 'product_limit',
				'type' => 'number',
				'default_value' => $this->config->get('ka_multivendor_default_products_limit'),
				'tooltip'    => $this->language->get('txt_tooltip_product_limit'),
				'sort_order' => 110,
				'section'    => $this->language->get('Vendor Limits')
			),
			'store_commission' => array(
				'code' => 'store_commission',
				'sort_order' => 120,
				'type'       => 'number',
				'default_value' => $this->config->get('ka_multivendor_default_commision'),
				'step'       => '0.01',
				'tooltip'    => $this->language->get('txt_tooltip_store_commission'),
			),
		);
		
		$custom_fields = $this->getCustomFields();
		
		foreach ($custom_fields as $custom_field) {
			if (!in_array($custom_field['location'], array('v_acc'))) {
				continue;
			}
			
			$custom_field['code']  = 'cf' . $custom_field['custom_field_id'];
			$custom_field['title'] = $custom_field['name'];
			$custom_field['is_vendor_custom'] = true;
			
			if (!empty($custom_field['custom_field_value'])) {
				$options = array();
				foreach ($custom_field['custom_field_value'] as $k => $v) {
					$options[$v['custom_field_value_id']] = $v['name'];
				}
				$custom_field['options'] = $options;
			}
			
			$custom_field['name']  = 'vendor_custom_field[' . $custom_field['custom_field_id'] . ']';
			$vendor_fields[$custom_field['code']] = $custom_field;
		}

		// sort vendor fields
		//
		uasort($vendor_fields, array($this, "sortArraysBySortOrder"));
		
		reset($vendor_fields);
		$fkey = key($vendor_fields);
		$vendor_fields[$fkey]['section'] = $this->language->get('Vendor Details');
		
		$fields = array_merge($fields, $vendor_fields);
		
		$profile_fields = array(
			'vendor_logo' => array(
				'tab' => 'public',
				'code' => 'vendor_logo',
				'type' => 'image',
				'required' => false,
				'sort_order' => 10,
			),
		);
		
		$fields = array_merge($fields, $profile_fields);
		
		return $fields;
	}

	public function addVendor($data) {

		$customer_id = parent::addVendor($data);
		if (empty($customer_id)) {
			return false;
		}
		
		if (!empty($data['vendor_status'])) {
			$this->setVendorStatus($customer_id, $data['vendor_status']);
		}
	}

	
	public function editVendor($customer_id, $data) {

		// regular vendor profile update
		//
		if (!parent::editVendor($customer_id, $data)) {
			return false;
		}
		
		$rec = array();
		$rec['product_limit']     = (int)(isset($data['product_limit']) ? $data['product_limit'] : $this->config->get('ka_multivendor_default_products_limit'));
		$rec['store_commission']  = (float)(isset($data['store_commission'])) ? $data['store_commission'] : $this->config->get('ka_multivendor_default_commision');
		$this->tbl_ka_vendor->update($data, "customer_id = '$customer_id'");
		
		
		// update vendor status
		//
		if (isset($data['vendor_status'])) {
			$this->setVendorStatus($customer_id, $data['vendor_status']);
		}
		
		return true;
	}	
	
	
	protected function setVendorStatus($customer_id, $status) {
	
		$status = (int)$status;
	
		$old_vendor_status = $this->getVendorStatus($customer_id);
		
		if ($status == $old_vendor_status) {
			return false;
		}

		// update vendor fields avaialble to admin only
		//
		$rec = array(
			'vendor_status' => $status,
		);
		$this->tbl_ka_vendor->update($rec, "customer_id = '$customer_id'");
		
		if (empty($status)) {
			$this->recordAction($customer_id, 'vendor_disabled');
		} else {
			$this->recordAction($customer_id, 'vendor_enabled');
		}
	}
	
	
	public function getCustomFields($customer_group_id = null) {
	
		if (empty($customer_group_id)) {
			$customer_group_id = $this->config->get('config_customer_group_id');		
		}
	
		// Custom Fields
		$this->load->model('customer/custom_field');

		$filter_data = array(
			'filter_customer_group_id' => $this->config->get('config_customer_group_id'),
			'sort'  => 'cf.sort_order',
			'order' => 'ASC'
		);
		
		$custom_fields = $this->model_customer_custom_field->getCustomFields($filter_data);
		
		if (empty($custom_fields)) {
			return $custom_fields;
		}
		
		foreach ($custom_fields as &$cf) {
			if ($cf['type'] == 'select' || $cf['type'] == 'radio' || $cf['type'] == 'checkbox') {
				$cf['custom_field_value'] = $this->model_customer_custom_field->getCustomFieldValues($cf['custom_field_id']);
			}
		}

		return $custom_fields;	
	}
}

class_alias(__NAMESPACE__ . '\ModelVendor', '\ModelExtensionKaExtensionsKaMultivendorVendor');