<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 139 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerAccountOrder extends \ControllerAccountOrder {

	use \extension\ka_extensions\TraitController;

	public function info() {

		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/customer/order');
		$kamodel_order_status = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order_status');

		$is_mv_order = false;
		if ($kamodel_order->isMultivendorOrder($this->request->get['order_id'])) {
			$is_mv_order = true;
		}
		
		$this->load->disableRender();
		parent::info();
		$this->load->enableRender();
		
		$data     = $this->getRenderData();
		$template = $this->getRenderTemplate();
		
		if (empty($data['products']) || !$is_mv_order) {
			return $this->response->setOutput($this->load->view($template, $data));
		}

		$data['vendor_orders'] = $this->getVendorOrders($this->request->get['order_id'], $data);
		
		if (!empty($data['vendor_orders'])) {
			foreach ($data['vendor_orders'] as &$vo) {
				$vo['delivery_status'] = $kamodel_order_status->getOrderStatus($vo['delivery_status_id']);
			}
		}
		
		return $this->response->setOutput($this->load->view($template, $data));
	}


	protected function getVendorOrders($order_id, $data) {

	
	
		$order_info = $this->model_account_order->getOrder($order_id);

		// arrange products by vendor
		// the vendor was taken from the order data here
		$products = array();
		foreach ($data['products'] as $p) {
			$products[$p['vendor_id']][] = $p;
		}

		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/customer/order');
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		
		// get vendor orders and fill them in with necessary information
		//
		$orders = $kamodel_order->getVendorOrdersByMainOrder($order_id);
		
		$vendor_orders = array();
		foreach ($orders as $vo) {
		
			if (!empty($vo['delivery_status_id']) 
				&& $vo['delivery_status_id'] != $this->config->get('ka_multivendor_confirm_status')) 
			{
				$vo['is_waiting_for_confirmation'] = true;
			}
		
			$vo['products']    = $products[$vo['vendor_id']];
			$vo['vendor_info'] = $kamodel_vendor->getVendor($vo['vendor_id']);

			$shipping_cost = $this->currency->format($vo['shipping_cost'], $order_info['currency_code'], $order_info['currency_value']);
			
			$totals = array();
			
			// add the shipping cost to totals
			//
			if (!empty($vo['shipping_code'])) {
				$total = array(
					'title' => str_replace('%shipping%', $vo['shipping_method'], $this->language->get('Shipping (%shipping%)')), 
					'text'  => $shipping_cost,
				);
				$totals['shipping'] = $total;
			}
			
			$vo['totals'] = $totals;
			
			$vendor_orders[$vo['vendor_id']] = $vo;
		}
		
		return $vendor_orders;
	}

	
	/*
		Ajax call from account/order/info page
	*/
	public function confirmDelivery() {
	
		$json = array();

		// process parameters
		//
		$params = array(
			'vendor_order_id' => 0
		);
		$params = array_replace($params, $this->request->get);

		// get vendor order details
		//
		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/customer/order');
		$vendor_order = $kamodel_order->getVendorOrder($params['vendor_order_id']);
		if (empty($vendor_order)) {
			$json['error'] = $this->language->get(sprintf('Vendor order #%s was not found', $params['vendor_order_id']));
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
		
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		$kamodel_order_status = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order_status');

		// change the delivery status
		//
		$confirm_status = $this->config->get('ka_multivendor_confirm_status');
		$comment = $this->language->get('Confirmed by the customer');
		
		$kamodel_order->changeVendorStatus($vendor_order['vendor_order_id'], $confirm_status, $comment, false);		
		
		// get an updated vendor section
		//
		$data = array();
		
		$data['delivery_statuses'] = $kamodel_order_status->getVendorOrderStatuses();
		
		$vendor_order = $kamodel_order->getVendorOrder($vendor_order['vendor_order_id']);

		$vendor_order['vendor_info']     = $kamodel_vendor->getVendor($vendor_order['vendor_id']);
		$vendor_order['delivery_status'] = $kamodel_order_status->getOrderStatus($vendor_order['delivery_status_id']);
		
		$data['vorder'] = $vendor_order;
		
		$html = $this->load->view('extension/ka_extensions/ka_multivendor/customer/order_vendor', $data);
		
		// return result to the page
		//
		$json['html'] = $html;
		
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
		return;
	}	
}