<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 122 $)
*/

namespace extension\ka_extensions\ka_multivendor;

/*
	session->data['shipping_method_by_vendor'] - array of a selected shipping quotes by vendors
	session->data['shipping_methods_by_vendor'] - array of a shipping methods by vendors
*/
class ControllerApiShipping extends \ControllerApiShipping {

	/*
		Calculates shipping methods by vendors into session
	*/
	public function methods() {

		if (!$this->cart->isMultivendorCart()) {
			return parent::methods();
		}
	
		$this->load->language('api/shipping');

		// Delete past shipping methods and method just in case there is an error
		unset($this->session->data['shipping_methods']);
		unset($this->session->data['shipping_method']);

		$this->session->data['shipping_methods_by_vendor'] = array();
		$this->session->data['shipping_method_by_vendor']  = array();
		
		$json = array();

		/*
			Check for errors
		*/
		if (!isset($this->session->data['api_id'])) {
			$json['error'] = $this->language->get('error_permission');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
			
			
		if (!$this->cart->hasShipping()) {
			$json['shipping_methods'] = array();

			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
		
		if (!isset($this->session->data['shipping_address'])) {
			$json['error'] = $this->language->get('error_address');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}

		$this->load->model('setting/extension');
		
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		
		$vendors = $this->cart->getVendors();

		// for existing orders we will retrieve the original shipping cost and put it together with new rates
		$order_id = 0;
		if (!empty($this->request->get['order_id'])) {
			$order_id = (int) $this->request->get['order_id'];
		}
		
		$no_shipping = false;
		foreach ($vendors as $vendor_id => $v) {
			$v = array_merge($v, $this->getVendorQuote($vendor_id, $order_id));

			if (!empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
				$v['selected_method'] = $this->session->data['shipping_method_by_vendor'][$vendor_id]['code'];
			} else {
				if (!empty($order_id) && !empty($v['shipping_methods']['original'])) {
					$v['selected_method'] = 'original.original';					
					$this->session->data['shipping_method_by_vendor'][$vendor_id] = $v['shipping_methods']['original']['quote']['original'];
				}
			}
			
			$vendors[$vendor_id] = array_merge($v, $kamodel_vendor->getVendor($vendor_id));
			
			$this->session->data['shipping_methods_by_vendor'][$vendor_id] = $v['shipping_methods'];
		}
		
		$json['shipping_methods_by_vendor'] = $vendors;
		
		if (empty($this->session->data['shipping_methods_by_vendor'])) {
			$json['error'] = $this->language->get('error_no_shipping');
		} else {
			$json['error'] = '';
		}
		
		$json['shipping_methods'] = array();
		
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
	
	
	protected function getVendorQuote($vendor_id, $order_id = 0) {
	
		$json = array();

		$this->cart->setVendorFilter($vendor_id);
		
		// Shipping Methods
		$json['shipping_methods'] = array();

		$results = $this->model_setting_extension->getExtensions('shipping');

		foreach ($results as $result) {
			if ($this->config->get('shipping_' . $result['code'] . '_status')) {
				$this->load->model('extension/shipping/' . $result['code']);

				$quote = $this->{'model_extension_shipping_' . $result['code']}->getQuote($this->session->data['shipping_address']);

				if ($quote) {
					$json['shipping_methods'][$result['code']] = array(
						'title'      => $quote['title'],
						'quote'      => $quote['quote'],
						'sort_order' => $quote['sort_order'],
						'error'      => $quote['error']
					);
				}
			}
		}
		$this->cart->unsetVendorFilter();

		// add the original shipping method from the order		
		if (!empty($order_id)) {
			$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
			$vorder = $kamodel_order->getVendorOrderByMainOrder($order_id, $vendor_id);
			if (!empty($vorder)) {
				$json['shipping_methods']['original'] = array(
					'title' => $this->language->get('Original'),
					'quote' => array(
						'original' => array(
							'code'  => 'original.original',
							'title' =>  $this->language->get('Original') . ' (' . $vorder['shipping_method'] . ')',
							'cost'  => $vorder['shipping_cost'],
							'tax_class_id' => 0,
							'text'  => $this->currency->format($vorder['shipping_cost'], $this->session->data['currency'])
						),
					),
					'sort_order' => -9999,
				);
			}
		}
		
		$sort_order = array();

		foreach ($json['shipping_methods'] as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $json['shipping_methods']);

		if ($json['shipping_methods']) {
			$this->session->data['shipping_methods'] = $json['shipping_methods'];
		} else {
			$json['error'] = $this->language->get('error_no_shipping');
		}

		return $json;
	}
	
	/*
		Called on submitting multive shipping methods from the order edit page 
	*/
	public function method_mv() {
		$this->load->language('api/shipping');

		// Delete old shipping method so not to cause any issues if there is an error
		$this->session->data['shipping_method_by_vendor'] = array();

		$json = array();

		if (!isset($this->session->data['api_id'])) {
			$json['error'] = $this->language->get('error_permission');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
		
		
		if (!$this->cart->hasShipping()) {
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
		
		// Shipping Address
		if (!isset($this->session->data['shipping_address'])) {
			$json['error'] = $this->language->get('error_address');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}

		// Shipping Method
		if (empty($this->session->data['shipping_methods_by_vendor'])) {
			$json['error'] = $this->language->get('error_no_shipping');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}
					
		if (!isset($this->request->post['shipping_method_by_vendor'])) {
			$json['error'] = $this->language->get('error_method');
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}

		// save shipping methods by vendors in a variable
		// $this->session->data['shipping_by_vendor']
		//
		$this->session->data['shipping_method_by_vendor'] = array();
		foreach ($this->request->post['shipping_method_by_vendor'] as $k => $v) {
		
			$shipping = explode('.', $v);
			
			if (!empty($this->session->data['shipping_methods_by_vendor'][$k][$shipping[0]]['quote'][$shipping[1]])) {
				$this->session->data['shipping_method_by_vendor'][$k] = $this->session->data['shipping_methods_by_vendor'][$k][$shipping[0]]['quote'][$shipping[1]];
			} else {
				$json['warning'] = $this->language->get('error_shipping');
			}			
		}

		if (empty($json)) {
			$json['success'] = $this->language->get('text_method');
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}