<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 119 $)
	
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerCheckoutCart extends \ControllerCheckoutCart { 

	use \extension\ka_extensions\TraitController;

	function onload() {
		// load langauge files
		$this->load->language('account/account');
		$this->load->language('extension/ka_extensions/ka_multivendor/common');
		$this->load->kamodel('extension/ka_extensions/ka_multivendor/shipping');
	}
	
	/*
		Replaced the standard cart page
	*/
	function index() {

		if (!$this->cart->isMultivendorCart()) {
			return parent::index();
		}
	
		$this->disableRender();
		parent::index();
		$this->enableRender();
		
		$data     = $this->getRenderData();
		$template = $this->getRenderTemplate();
		
		if (empty($data['products'])) {
			return $this->response->setOutput($this->load->view($template, $data));
		}
	
		// copy messages to top_messages
		//
		if (!empty($data['success'])) {
			$this->addTopMessage($data['success']);
		} 
		
		if (!empty($data['error_warning'])) {
			$this->addTopMessage($data['error_warning'], 'E');
		}
		
		if (!empty($data['info'])) {
			$this->addTopMessage($data['info'], 'I');
		}
			
		$vendors = array();
		
		// add products to the vendors array
		$vendors = $this->fillProductsByVendors($vendors, $data['products']);

		// add shipping and other info to the vendors array
		$vendors = $this->addInfoByVendors($vendors);

		$data['vendors'] = $vendors;

		$this->language->load('checkout/cart');
		
		$this->data = $data;
		$this->template = 'extension/ka_extensions/ka_multivendor/cart';
		
		$this->response->setOutput($this->render());
	}

	
	protected function fillProductsByVendors($vendors, $products) {
	
		// get products by vendors
		//
		foreach ($products as $p) {
		
			$vendor_id = $p['vendor_id'];
		
			if (empty($vendors[$vendor_id])) {
				$vendors[$vendor_id] = array(
					'products'  => array(),
					'subtotal'  => null
				);
			}
			
			// Display prices
			if ($this->customer->isLogged() || !$this->config->get('config_customer_price')) {
				$total = $p['quantity'] * $p['unit_price'];
				$vendors[$vendor_id]['subtotal'] += $total;
			}
			
			$vendors[$vendor_id]['products'][] = $p;
		}

		return $vendors;
	}
	
	
	protected function addInfoByVendors($vendors) {
	
		// update information by vendors
		//
		foreach ($vendors as $vendor_id => $v) {
			$vendors[$vendor_id] = $this->addInfoForVendor($vendor_id, $v);
		}

		return $vendors;
	}
	
	
	protected function addInfoForVendor($vendor_id, $vendor) {

		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
	
		$totals = array();
		
		// add subtotal to totals
		//
		if (!is_null($vendor['subtotal'])) {
		
			$total = array(
				'title' => $this->language->get('Subtotal'),
				'text'  => $this->currency->format($vendor['subtotal'], $this->session->data['currency'])
			);
			
			$totals['subtotal'] = $total;				
		}
		
		// add shipping cost
		if (!empty($this->session->data['shipping_method_by_vendor'])) {
			if (!empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
				$shipping = $this->session->data['shipping_method_by_vendor'][$vendor_id];

				$total = array(
					'title' => str_replace('%shipping%', $shipping['title'], $this->language->get('Shipping (%shipping%)')), 
					'text'  => $this->currency->format($shipping['cost'], $this->session->data['currency'])
				);
				$totals['shipping'] = $total;
			}
		}
		
		$vendor['totals'] = $totals;
		
		$vendor['info'] = $kamodel_vendor->getVendor($vendor_id);
		
		return $vendor;
	}	
}