<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 154 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerCheckoutShippingMethod extends \ControllerCheckoutShippingMethod {

	public function index() {
	
		if (!$this->cart->isMultivendorCart()) {
			return parent::index();
		}	
	
		$this->load->language('checkout/checkout');
		$this->load->model('setting/extension');
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		
		$this->session->data['shipping_methods_by_vendors'] = array();
		
		// Shipping Methods
		$errors = array();
		if (isset($this->session->data['shipping_address'])) {

			$vendors = $this->cart->getVendors();
			
			$this->session->data['shipping_methods_by_vendor'] = array();
			
			foreach ($vendors as $vendor_id => $v) {
				
				$vendor_info = $kamodel_vendor->getVendor($vendor_id);
					
				$this->cart->setVendorFilter($vendor_id);
				
				if ($this->cart->hasShipping()) {
					$v['shipping_methods'] = $this->getShippingMethodsForCart();

					if (empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
						if (!empty($v['shipping_methods'])) {
							// try to set the first available method for the vendor
							// when th select method is not specified
							$methods = reset($v['shipping_methods']);
							$quotes  = reset($methods['quote']);
							if (!empty($quotes['code'])) {
								$v['selected_method'] = $quotes['code'];
								$code = explode('.', $quotes['code']);
								if (!empty($this->session->data['shipping_methods_by_vendor'][$vendor_id])) {
									$this->session->data['shipping_method_by_vendor'][$vendor_id] = $this->session->data['shipping_methods_by_vendor'][$vendor_id][$code[0]]['quote'][$code[1]];
								}
							}
						}
					}

					if (!empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
						$v['selected_method'] = $this->session->data['shipping_method_by_vendor'][$vendor_id]['code'];
					}				
					
					$vendors[$vendor_id] = array_merge($v, $vendor_info);
					
					$this->session->data['shipping_methods_by_vendor'][$vendor_id] = $v['shipping_methods'];

				} else {

					// the vendor may require no shipping for electronic goods
					//
					$vendors[$vendor_id] = array_merge($v, $vendor_info);

					$vendors[$vendor_id]['has_no_shipping'] = true;
				
					$this->session->data['shipping_methods_by_vendor'][$vendor_id] = array(
						'has_no_shipping' => true
					);
				}
				
				$this->cart->unsetVendorFilter();				
			}			
		}
		
		$data['shipping_methods_by_vendor'] = $vendors;
		
		// when the 'Shipping Methods' section shows we assume that the cart requires a shipping method
		//
		if (empty($this->session->data['shipping_methods_by_vendor'])) {
			$data['error_warning'] = sprintf($this->language->get('error_no_shipping'), $this->url->link('information/contact'));
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->session->data['comment'])) {
			$data['comment'] = $this->session->data['comment'];
		} else {
			$data['comment'] = '';
		}
		
		$this->response->setOutput($this->load->view('extension/ka_extensions/ka_multivendor/shipping_method', $data));
	}
	

	public function save() {
		if (!$this->cart->isMultivendorCart()) {
			return parent::save();
		}	
	
		$this->load->language('checkout/checkout');

		$json = array();

		// Validate if shipping is required. If not the customer should not have reached this page.
		if (!$this->cart->hasShipping()) {
			$json['redirect'] = $this->url->link('checkout/checkout', '', true);
		}

		// Validate if shipping address has been set.
		if (!isset($this->session->data['shipping_address'])) {
			$json['redirect'] = $this->url->link('checkout/checkout', '', true);
		}

		// Validate cart has products and has stock.
		if ((!$this->cart->hasProducts() && empty($this->session->data['vouchers'])) || (!$this->cart->hasStock() && !$this->config->get('config_stock_checkout'))) {
			$json['redirect'] = $this->url->link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = $this->cart->getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$json['redirect'] = $this->url->link('checkout/cart');

				break;
			}
		}
		
		// check if the array is available
		//
		if (empty($this->request->post['shipping_method_by_vendor'])) {
			$json['error']['warning'] = $this->language->get('error_shipping');
		
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}

		// save shipping method by vendors in a variable
		// $this->session->data['shipping_method_by_vendor']
		//
		$this->session->data['shipping_method_by_vendor'] = array();
		$vendors = $this->cart->getVendors();		

		foreach ($vendors as $vendor_id => $v) {

			// skip the vendor if it does not require the shipping
			//
			$this->cart->setVendorFilter($vendor_id);
			if (!$this->cart->hasShipping()) {
				$this->cart->unsetVendorFilter();
				$vendors[$vendor_id]['has_no_shipping'] = true;
				$this->session->data['shipping_methods_by_vendor'][$vendor_id] = array(
					'has_no_shipping' => true,
				);
				continue;
			}
			$this->cart->unsetVendorFilter();
		
			if (empty($this->request->post['shipping_method_by_vendor'][$vendor_id])) {
				$json['error']['warning'] = $this->language->get('error_shipping');
				break;
			}
						
			$shipping = explode('.', $this->request->post['shipping_method_by_vendor'][$vendor_id]);
			
			if (!empty($this->session->data['shipping_methods_by_vendor'][$vendor_id][$shipping[0]]['quote'][$shipping[1]])) {
				$this->session->data['shipping_method_by_vendor'][$vendor_id] = $this->session->data['shipping_methods_by_vendor'][$vendor_id][$shipping[0]]['quote'][$shipping[1]];
			} else {
				$json['error']['warning'] = $this->language->get('error_shipping');
			}			
		}
		
		if (!$json) {
			$this->session->data['comment'] = strip_tags($this->request->post['comment']);
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
	
	
	/*
		This is a copy from shipping_quote. It can probably be moved to a common model file.
	*/	
	protected function getShippingMethodsForCart() {

		$quote_data = array();		

		$results = $this->model_setting_extension->getExtensions('shipping');
		
		foreach ($results as $result) {
			if ($this->config->get('shipping_' . $result['code'] . '_status')) {
				$this->load->model('extension/shipping/' . $result['code']);

				$quote = $this->{'model_extension_shipping_' . $result['code']}->getQuote($this->session->data['shipping_address']);

				if ($quote) {
					$quote_data[$result['code']] = array(
						'title'      => $quote['title'],
						'quote'      => $quote['quote'],
						'sort_order' => $quote['sort_order'],
						'error'      => $quote['error']
					);
				}
			}
		}
		
		$sort_order = array();

		foreach ($quote_data as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $quote_data);
		
		return $quote_data;
	}
}