<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 147 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerProfileEdit extends profile\AControllerVendor {

	protected function onLoad() {
		$this->load->model('tool/image');
		
		return parent::onLoad();
	}

	/*
		Vendor edit page
	*/
	public function index() {

		$vendor_status = $this->customer->isVendor();

		$this->load->language('account/register');
		$this->load->language('extension/ka_extensions/ka_multivendor/profile');
		
		if (empty($vendor_status)) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}

		$this->document->setTitle($this->language->get('heading_title'));

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$this->kamodel_vendor->editVendor($this->customer->getId(), $this->request->post);
			$this->addTopMessage($this->language->get('txt_saved_successfully'));
			$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/profile/edit'));
		}
		
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();
		$this->data['languages'] = $languages;
		
		// breadcrumbs
		//
		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('txt_heading_title'),
			'href' => $this->url->link('extension/ka_extensions/ka_multivendor/profile/edit', '', true)
		);
		$this->data['text_account_already'] = sprintf($this->language->get('text_account_already'), $this->url->link('account/login', '', true));

		// error processing
		//
		if (isset($this->errors['warning'])) {
			$this->addTopMessage($this->errors['warning'], 'E');
		}
		
		$vendor = $this->kamodel_vendor->getVendor();
		$vendor_description = $this->kamodel_vendor->getVendorDescription();
		
		$fields = $this->kamodel_vendor->getAccountFields('E');
		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['fields'] = $this->getFieldsWithData($fields, $vendor, $this->request->post, $this->errors);
		} else {
			$this->data['fields'] = $this->getFieldsWithData($fields, $vendor);
		}
		
		// get fields for the vendor description
		//
		$this->data['vd_fields'] = array();
		$fields = $this->kamodel_vendor->getVendorDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['vd_fields'] = $this->getLanguageFieldsWithData('vendor_description', $fields, $vendor_description, $this->request->post, $this->errors);
		} else {
			$this->data['vd_fields'] = $this->getLanguageFieldsWithData('vendor_description', $fields, $vendor_description);
		}
		
		// page data preparation
		//
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/profile/edit', '', true);
		
		// Captcha
		if ($this->config->get('captcha_' . $this->config->get('config_captcha') . '_status') && in_array('register', (array)$this->config->get('config_captcha_page'))) {
			$this->data['captcha'] = $this->load->controller('extension/captcha/' . $this->config->get('config_captcha'), $this->errors);
		} else {
			$this->data['captcha'] = '';
		}
		
		if ($this->config->get('config_account_id')) {
			$this->load->model('catalog/information');

			$information_info = $this->model_catalog_information->getInformation($this->config->get('config_account_id'));

			if ($information_info) {
				$this->data['text_agree'] = sprintf($this->language->get('text_agree'), $this->url->link('information/information/agree', 'information_id=' . $this->config->get('config_account_id'), true), $information_info['title'], $information_info['title']);
			} else {
				$this->data['text_agree'] = '';
			}
		} else {
			$this->data['text_agree'] = '';
		}
				
		if (isset($this->request->post['agree'])) {
			$this->data['agree'] = $this->request->post['agree'];
		} else {
			$this->data['agree'] = false;
		}

		$this->data['image_placeholder'] = $this->model_tool_image->resize('no_image.png', 100, 100);
		
		//
		// prepare the view
		//
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		$this->document->addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');

		$this->showPage('extension/ka_extensions/ka_multivendor/profile/edit');
	}

	/*
		Validate the post data from to vendor edit page
	*/
	protected function validate() {
	
		$fields = $this->kamodel_vendor->getAccountFields();
		
		foreach ($fields as $k => $f) {
			if (!isset($this->request->post[$k])) {
				$this->request->post[$k] = '';
			}
		}
		
		// Custom field validation
		$this->load->model('account/custom_field');

		$customer_group_id = $this->config->get('config_customer_group_id');
		$custom_fields = $this->model_account_custom_field->getVendorCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			if (!empty($custom_field['required']) && empty($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']])) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		
		// check vendor description fields
		//
		$fields = $this->kamodel_vendor->getVendorDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		if (!$this->validateLanguageFields('vendor_description', $fields, $this->request->post)) {
			$has_errors = true;
		}

      	if (!empty($this->errors)) {
      		$this->addTopMessage($this->language->get("text_invalid_form_data"), 'E');
      		return false;
      	}
		
		return true;
	}
}

class_alias(__NAMESPACE__ . '\ControllerProfileEdit', '\ControllerExtensionKaExtensionsKaMultivendorProfileEdit');