<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 146 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerProfileRegister extends profile\AControllerVendor {

	protected function onLoad() {
		$this->load->language('extension/ka_extensions/ka_multivendor/profile');
		return parent::onLoad();
	}

	/*
		Vendor edit page
	*/
	public function index() {
	
		$vendor_status = $this->kamodel_vendor->getVendorStatus();
		
		if (!$this->customer->getId() || !is_null($vendor_status)) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
		
		$this->load->language('account/register');

		$fields = $this->kamodel_vendor->getAccountFields('V');
		
		$this->document->setTitle($this->language->get('heading_title'));

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$vendor_id = $this->kamodel_vendor->addVendor($this->request->post);

			$this->addTopMessage($this->language->get('You have registered a vendor account'));
			$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/profile/edit'));
		}

		// breadcrumbs
		//
		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_register'),
			'href' => $this->url->link('extension/ka_extensions/ka_multivendor/profile/register', '', true)
		);
		$this->data['text_account_already'] = sprintf($this->language->get('text_account_already'), $this->url->link('account/login', '', true));

		// error processing
		//
		if (isset($this->errors['warning'])) {
			$this->addTopMessage($this->errors['warning'], 'E');
		}
		
		$fields = $this->kamodel_vendor->getAccountFields('V');
		$this->data['fields'] = $this->getFieldsWithData($fields, array(), $this->request->post, $this->errors);
		
		// page data preparation
		//
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/profile/register', '', true);
		
		if ($this->config->get('config_account_id')) {
			$this->load->model('catalog/information');

			$information_info = $this->model_catalog_information->getInformation($this->config->get('config_account_id'));

			if ($information_info) {
				$this->data['text_agree'] = sprintf($this->language->get('text_agree'), $this->url->link('information/information/agree', 'information_id=' . $this->config->get('config_account_id'), true), $information_info['title'], $information_info['title']);
			} else {
				$this->data['text_agree'] = '';
			}
		} else {
			$this->data['text_agree'] = '';
		}

		if (isset($this->request->post['agree'])) {
			$this->data['agree'] = $this->request->post['agree'];
		} else {
			$this->data['agree'] = false;
		}

		//
		// prepare the view
		//
				
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		$this->document->addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		$this->document->addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');

		$this->showPage('extension/ka_extensions/ka_multivendor/profile/register');
	}

	/*
		Validate the post data from to vendor edit page
	*/
	protected function validate() {
	
		$fields = $this->kamodel_vendor->getAccountFields();
		
		foreach ($fields as $k => $f) {
			if (!isset($this->request->post[$k])) {
				$this->request->post[$k] = '';
			}
		}
	
		// Custom field validation
		$this->load->model('account/custom_field');
		
		$customer_group_id = $this->config->get('config_customer_group_id');

		$custom_fields = $this->model_account_custom_field->getVendorCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['required'] && empty($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']])) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		
		return !$this->errors;
	}
}

class_alias(__NAMESPACE__ . '\ControllerProfileRegister', '\ControllerExtensionKaExtensionsKaMultivendorProfileRegister');