<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 146 $)
	
	
	This class is used for a customer registration form. It will register vendor and customer at once.
	
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerRegister extends profile\AControllerVendor {

	/*
		Vendor registration page and processing the action
	*/
	public function index() {
		$vendor_status = $this->customer->isVendor();
	
		if (!empty($vendor_status)) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}

		$this->load->language('account/register');
		$this->load->language('extension/ka_extensions/ka_multivendor/profile');

		$this->document->setTitle($this->language->get('heading_title'));

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$vendor_id = $this->kamodel_vendor->addVendor($this->request->post);
			
			// Clear any previous login attempts for unregistered accounts.
			$this->model_account_customer->deleteLoginAttempts($this->request->post['email']);

			$this->customer->login($this->request->post['email'], $this->request->post['password']);

			unset($this->session->data['guest']);

			$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/register/registered'));
		}

		//
		// prepare the view
		//
		
		$fields = $this->kamodel_vendor->getAccountFields('R');
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['fields'] = $this->getFieldsWithData($fields, array(), $this->request->post, $this->errors);
		} else {
			$this->data['fields'] = $this->getFieldsWithData($fields, array());
		}		

		// breadcrumbs
		//
		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_register'),
			'href' => $this->url->link('account/register', '', true)
		);
		$this->data['text_account_already'] = sprintf($this->language->get('text_account_already'), $this->url->link('account/login', '', true));

		// error processing
		//
		if (isset($this->errors['warning'])) {
			$this->addTopMessage($this->errors['warning'], 'E');
		}
		
		// page data preparation
		//
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/register', '', true);

		// Captcha
		if ($this->config->get('captcha_' . $this->config->get('config_captcha') . '_status') && in_array('register', (array)$this->config->get('config_captcha_page'))) {
			$this->data['captcha'] = $this->load->controller('extension/captcha/' . $this->config->get('config_captcha'), $this->errors);
		} else {
			$this->data['captcha'] = '';
		}

		if ($this->config->get('config_account_id')) {
			$this->load->model('catalog/information');

			$information_info = $this->model_catalog_information->getInformation($this->config->get('config_account_id'));

			if ($information_info) {
				$this->data['text_agree'] = sprintf($this->language->get('text_agree'), $this->url->link('information/information/agree', 'information_id=' . $this->config->get('config_account_id'), true), $information_info['title'], $information_info['title']);
			} else {
				$this->data['text_agree'] = '';
			}
		} else {
			$this->data['text_agree'] = '';
		}

		if (isset($this->request->post['agree'])) {
			$this->data['agree'] = $this->request->post['agree'];
		} else {
			$this->data['agree'] = false;
		}

		$this->showPage('extension/ka_extensions/ka_multivendor/register');
	}

	/*
		Validate the post data from to vendor registration page
	*/
	protected function validate() {
	
		$fields = $this->kamodel_vendor->getAccountFields('R');

		// check firstname
		//
		if (empty($this->request->post['firstname']) 
			|| (utf8_strlen(trim($this->request->post['firstname'])) < 1) || (utf8_strlen(trim($this->request->post['firstname'])) > 32)
		) {
			$this->errors['firstname'] = $this->language->get('error_firstname');
		}

		// check lastname
		//
		if (empty($this->request->post['lastname']) 
			|| (utf8_strlen(trim($this->request->post['lastname'])) < 1) || (utf8_strlen(trim($this->request->post['lastname'])) > 32)) 
		{
			$this->errors['lastname'] = $this->language->get('error_lastname');
		}

		// check email
		//
		if (empty($this->request->post['email']) 
			|| (utf8_strlen($this->request->post['email']) > 96) || !filter_var($this->request->post['email'], FILTER_VALIDATE_EMAIL)
		) {
			$this->errors['email'] = $this->language->get('error_email');
		}

		// check email existance
		//
		if ($this->model_account_customer->getTotalCustomersByEmail($this->request->post['email'])) {
			$this->errors['warning'] = $this->language->get('error_exists');
		}
		
		// check the phone
		//
		if (empty($this->request->post['telephone']) 
			|| (utf8_strlen($this->request->post['telephone']) < 3) || (utf8_strlen($this->request->post['telephone']) > 32)
		) {
			$this->errors['telephone'] = $this->language->get('error_telephone');
		}
		
		// Customer Group
		if (isset($this->request->post['customer_group_id']) && is_array($this->config->get('config_customer_group_display')) && in_array($this->request->post['customer_group_id'], $this->config->get('config_customer_group_display'))) {
			$customer_group_id = $this->request->post['customer_group_id'];
		} else {
			$customer_group_id = $this->config->get('config_customer_group_id');
		}

		// profile custom field validation
		//
		$this->load->model('account/custom_field');
		$customer_group_id = $this->config->get('config_customer_group_id');
		$custom_fields = $this->model_account_custom_field->getCustomFields($customer_group_id);
		foreach ($custom_fields as $custom_field) {
			
			if ($custom_field['location'] != 'account') {
				continue;
			}
			
			if ($custom_field['required'] 
				&& empty($this->request->post['custom_field'][$custom_field['custom_field_id']])
			) {
				$this->errors['custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			} elseif (
				($custom_field['type'] == 'text') 
				&& !empty($custom_field['validation']) 
				&& !filter_var($this->request->post['custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))
			) {				
				$this->errors['custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		
		// vendor custom field validation
		//
		$customer_group_id = $this->config->get('config_customer_group_id');
		$custom_fields = $this->model_account_custom_field->getVendorCustomFields($customer_group_id);
		foreach ($custom_fields as $custom_field) {
			if ($custom_field['required'] 
				&& empty($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']])
			) {
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			} elseif (
				($custom_field['type'] == 'text') 
				&& !empty($custom_field['validation']) 
				&& !filter_var($this->request->post['vendor_custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))
			) {				
				$this->errors['vendor_custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		
		
		if (empty($this->request->post)
			|| (utf8_strlen(html_entity_decode($this->request->post['password'], ENT_QUOTES, 'UTF-8')) < 4) || (utf8_strlen(html_entity_decode($this->request->post['password'], ENT_QUOTES, 'UTF-8')) > 40)
		) {
			$this->errors['password'] = $this->language->get('error_password');
		}

		if (empty($this->request->post['confirm']) || $this->request->post['confirm'] != $this->request->post['password']) {
			$this->errors['confirm'] = $this->language->get('error_confirm');
		}

		// Captcha
		if ($this->config->get('captcha_' . $this->config->get('config_captcha') . '_status') && in_array('register', (array)$this->config->get('config_captcha_page'))) {
			$captcha = $this->load->controller('extension/captcha/' . $this->config->get('config_captcha') . '/validate');

			if ($captcha) {
				$this->errors['captcha'] = $captcha;
			}
		}

		// Agree to terms
		//
		if ($this->config->get('config_account_id')) {
			$this->load->model('catalog/information');

			$information_info = $this->model_catalog_information->getInformation($this->config->get('config_account_id'));

			if ($information_info && !isset($this->request->post['agree'])) {
				$this->errors['warning'] = sprintf($this->language->get('error_agree'), $information_info['title']);
			}
		}
		
		return !$this->errors;
	}

	/*
		Show this page after successfully submitted vendor account
		
		It will still require approval from admin
	*/
	public function registered() {
		$this->load->language('account/success');
		$this->load->language('extension/ka_extensions/ka_multivendor/profile');

		$this->document->setTitle($this->language->get('heading_title'));

		// breadcrumbs
		//
		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_success'),
			'href' => $this->url->link('account/success')
		);

		$this->data['text_message'] = sprintf($this->language->get('text_vendor_registered'), $this->url->link('information/contact'));

		$this->data['continue'] = $this->url->link('account/account', '', true);

		$this->data['column_left'] = $this->load->controller('common/column_left');
		$this->data['column_right'] = $this->load->controller('common/column_right');
		$this->data['content_top'] = $this->load->controller('common/content_top');
		$this->data['content_bottom'] = $this->load->controller('common/content_bottom');
		$this->data['footer'] = $this->load->controller('common/footer');
		$this->data['header'] = $this->load->controller('common/header');

		$this->response->setOutput($this->load->view('common/success', $this->data));
	}
}

class_alias(__NAMESPACE__ . '\ControllerRegister', '\ControllerExtensionKaExtensionsKaMultivendorRegister');