<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 155 $)

	shipping_methods_by_vendor - contains a list of possible methods for the vendor
	shipping_method_by_vendor - contains a particular quote selected by the customer for each vendor

*/
namespace extension\ka_extensions\ka_multivendor;

class ControllerShippingQuote extends \extension\ka_extensions\Controller {

	protected $kamodel_vendor;

	protected function onLoad() {
		$this->load->model('setting/extension');
		$this->kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		return parent::onLoad();
	}	

	/*
		This is called by ajax to show a list of shipping methods by vendors on a 'Shipping estimator' popup window.
		
		The function calculates rates by vendors and saves them into a session variable:
			$this->session->data['shipping_methods_by_vendor']
		
	*/
	public function quote() {

		$vendors = $this->cart->getVendors();
		
		$this->session->data['shipping_methods_by_vendor'] = array();

		$no_shipping = false;
		foreach ($vendors as $vendor_id => $v) {
		
			$this->cart->setVendorFilter($vendor_id);
			if (!$this->cart->hasShipping()) {
				$vendors[$vendor_id]['has_no_shipping'] = true;
				continue;
			}

			$v['shipping_methods'] = $this->getVendorQuote($vendor_id);
			
			
			if (!empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
				$v['selected_method'] = $this->session->data['shipping_method_by_vendor'][$vendor_id]['code'];
			}

			$vendors[$vendor_id] = array_merge($v, $this->kamodel_vendor->getVendor($vendor_id));
			
			$this->session->data['shipping_methods_by_vendor'][$vendor_id] = $v['shipping_methods'];
		}
		$this->cart->unsetVendorFilter();		

		$this->data['vendors'] = $vendors;

		$json['vendors_html'] = $this->render('extension/ka_extensions/ka_multivendor/shipping_quote');
		
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
	
	
	protected function getVendorQuote($vendor_id) {

		$this->cart->setVendorFilter($vendor_id);
		
		$quote_data = array();

		$results = $this->model_setting_extension->getExtensions('shipping');
		
		foreach ($results as $result) {

			if ($this->config->get('shipping_' . $result['code'] . '_status')) {
				$this->load->model('extension/shipping/' . $result['code']);

				$quote = $this->{'model_extension_shipping_' . $result['code']}->getQuote($this->session->data['shipping_address']);

				if ($quote) {
					$quote_data[$result['code']] = array(
						'title'      => $quote['title'],
						'quote'      => $quote['quote'],
						'sort_order' => $quote['sort_order'],
						'error'      => $quote['error']
					);
				}
			}
		}
		
		$sort_order = array();

		foreach ($quote_data as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $quote_data);

		$this->cart->unsetVendorFilter();
		
		return $quote_data;
	}
	
	/*
		Receives a submit from the shipping calculator on a cart page
		
		It is supposed to save preferred shipping methods by vendors
	*/
	public function shipping() {
		$this->load->language('extension/total/shipping');

		$json = array();

		// check if the array is available
		//
		if (empty($this->request->post['shipping_method_by_vendor'])) {
			$json['warning'] = $this->language->get('error_shipping');
		
			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($json));
			return;
		}

		// save shipping methods by vendors in a variable
		// $this->session->data['shipping_by_vendor']
		//
		$this->session->data['shipping_method_by_vendor'] = array();
		foreach ($this->request->post['shipping_method_by_vendor'] as $k => $v) {
		
			$shipping = explode('.', $v);
			
			if (!empty($this->session->data['shipping_methods_by_vendor'][$k][$shipping[0]]['quote'][$shipping[1]])) {
				$this->session->data['shipping_method_by_vendor'][$k] = $this->session->data['shipping_methods_by_vendor'][$k][$shipping[0]]['quote'][$shipping[1]];
				
				// here we unset the text variable because it does not match the cost and may confuse
				// later
				if (!empty($this->session->data['shipping_method_by_vendor'][$k]['text'])) {
					unset($this->session->data['shipping_method_by_vendor'][$k]['text']);
				}
			} else {
				$json['warning'] = $this->language->get('error_shipping');
			}			
		}
		
		$this->session->data['success'] = $this->language->get('text_success');
		$json['redirect'] = $this->url->link('checkout/cart');

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}

class_alias(__NAMESPACE__ . '\ControllerShippingQuote', 'ControllerExtensionKaExtensionsKaMultivendorShippingQuote');