<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 151 $)
*/

namespace extension\ka_extensions\ka_multivendor;

use extension\ka_extensions\UrlParams;

class ControllerVendor extends \extension\ka_extensions\Controller {

	protected function onLoad() {

		$this->load->model('catalog/product');
		$this->load->model('tool/image');
		$this->load->language('extension/ka_extensions/ka_multivendor/vendor');
	}
	
	public function index() {

	
		$params = array(
			'vendor_id' => 0,
			'sort'      => 'p.sort_order',
			'order'     => 'asc',
			'page'      => 1,
			'limit'     => (int)$this->config->get('theme_' . $this->config->get('config_theme') . '_product_limit'),
		);
		
		$url_params = new UrlParams($this->request, $params);
		$params = $url_params->getParams();
		
		$vendor_info = $this->kamodel_vendor->getVendor($params['vendor_id']);

		if ($vendor_info) {
		
			if (!empty($vendor_info['vendor_logo'])) {
				$data['thumb'] = $this->model_tool_image->resize($vendor_info['vendor_logo'], $this->config->get('theme_' . $this->config->get('config_theme') . '_image_category_width'), $this->config->get('theme_' . $this->config->get('config_theme') . '_image_category_height'));
			} else {
				$data['thumb'] = '';
			}
			
			if (!empty($vendor['description'])) {
				$data['description'] = $vendor_info['description'];
			}
			$data['name'] = $vendor_info['name'];
			
			$this->document->setTitle($vendor_info['name']);

			$url = $url_params->getUrl();

			$data['breadcrumbs'][] = array(
				'text' => $vendor_info['name'],
				'href' => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . $url)
			);

			$data['heading_title'] = $vendor_info['name'];

			$data['text_compare'] = sprintf($this->language->get('text_compare'), (isset($this->session->data['compare']) ? count($this->session->data['compare']) : 0));

			$data['compare'] = $this->url->link('product/compare');

			$data['products'] = array();

			$filter_data = array(
				'filter_vendor_id' => $params['vendor_id'],
				'sort'  => $params['sort'],
				'order' => $params['order'],
				'start'           => ($params['page'] - 1) * $params['limit'],
				'limit'           => $params['limit']
			);
				
			$product_total = $this->model_catalog_product->getTotalProducts($filter_data);

			$results = $this->model_catalog_product->getProducts($filter_data);

			foreach ($results as $result) {
				if ($result['image']) {
					$image = $this->model_tool_image->resize($result['image'], $this->config->get('theme_' . $this->config->get('config_theme') . '_image_product_width'), $this->config->get('theme_' . $this->config->get('config_theme') . '_image_product_height'));
				} else {
					$image = $this->model_tool_image->resize('placeholder.png', $this->config->get('theme_' . $this->config->get('config_theme') . '_image_product_width'), $this->config->get('theme_' . $this->config->get('config_theme') . '_image_product_height'));
				}

				if ($this->customer->isLogged() || !$this->config->get('config_customer_price')) {
					$price = $this->currency->format($this->tax->calculate($result['price'], $result['tax_class_id'], $this->config->get('config_tax')), $this->session->data['currency']);
				} else {
					$price = false;
				}

				if ((float)$result['special']) {
					$special = $this->currency->format($this->tax->calculate($result['special'], $result['tax_class_id'], $this->config->get('config_tax')), $this->session->data['currency']);
				} else {
					$special = false;
				}

				if ($this->config->get('config_tax')) {
					$tax = $this->currency->format((float)$result['special'] ? $result['special'] : $result['price'], $this->session->data['currency']);
				} else {
					$tax = false;
				}

				if ($this->config->get('config_review_status')) {
					$rating = (int)$result['rating'];
				} else {
					$rating = false;
				}

				$data['products'][] = array(
					'product_id'  => $result['product_id'],
					'thumb'       => $image,
					'name'        => $result['name'],
					'description' => utf8_substr(trim(strip_tags(html_entity_decode($result['description'], ENT_QUOTES, 'UTF-8'))), 0, $this->config->get('theme_' . $this->config->get('config_theme') . '_product_description_length')) . '..',
					'price'       => $price,
					'special'     => $special,
					'tax'         => $tax,
					'minimum'     => $result['minimum'] > 0 ? $result['minimum'] : 1,
					'rating'      => $result['rating'],
					'href'        => $this->url->link('product/product', 'vendor_id=' . $result['vendor_id'] . '&product_id=' . $result['product_id'] . $url)
				);
			}

			$url = '';

			if (isset($this->request->get['limit'])) {
				$url .= '&limit=' . $this->request->get['limit'];
			}

			$data['sorts'] = array();

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_default'),
				'value' => 'p.sort_order-ASC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=p.sort_order&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_name_asc'),
				'value' => 'pd.name-ASC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=pd.name&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_name_desc'),
				'value' => 'pd.name-DESC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=pd.name&order=DESC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_price_asc'),
				'value' => 'p.price-ASC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=p.price&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_price_desc'),
				'value' => 'p.price-DESC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=p.price&order=DESC' . $url)
			);

			if ($this->config->get('config_review_status')) {
				$data['sorts'][] = array(
					'text'  => $this->language->get('text_rating_desc'),
					'value' => 'rating-DESC',
					'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=rating&order=DESC' . $url)
				);

				$data['sorts'][] = array(
					'text'  => $this->language->get('text_rating_asc'),
					'value' => 'rating-ASC',
					'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=rating&order=ASC' . $url)
				);
			}

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_model_asc'),
				'value' => 'p.model-ASC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=p.model&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => $this->language->get('text_model_desc'),
				'value' => 'p.model-DESC',
				'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . '&sort=p.model&order=DESC' . $url)
			);

			$url = '';

			if (isset($this->request->get['sort'])) {
				$url .= '&sort=' . $this->request->get['sort'];
			}

			if (isset($this->request->get['order'])) {
				$url .= '&order=' . $this->request->get['order'];
			}

			$data['limits'] = array();

			$limits = array_unique(array($this->config->get('theme_' . $this->config->get('config_theme') . '_product_limit'), 25, 50, 75, 100));

			sort($limits);

			foreach($limits as $value) {
				$data['limits'][] = array(
					'text'  => $value,
					'value' => $value,
					'href'  => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . $url . '&limit=' . $value)
				);
			}

			$url = $url_params->getUrl();

			$pagination        = new \KaPagination();
			$pagination->total = $product_total;
			$pagination->page  = $params['page'];
			$pagination->limit = $params['limit'];
			$pagination->url   = $this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] .  $url . '&page={page}');
			
			$data['pagination'] = $pagination->render();
			$data['results']    = $pagination->getResults();

			if ($url_params->getParam('page') == 1) {
			    $this->document->addLink($this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'], true), 'canonical');
			} else {
				$this->document->addLink($this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . $url . '&page='. $page, true), 'canonical');
			}
			
			if ($url_params->getParam('page') > 1) {
			    $this->document->addLink($this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . $url . '&page='. (($page - 2) ? '&page='. ($page - 1) : ''), true), 'prev');
			}

			if ($params['limit'] && ceil($product_total / $params['limit']) > $params['page']) {
			    $this->document->addLink($this->url->link('extension/ka_extensions/ka_multivendor/vendor', 'vendor_id=' . $this->request->get['vendor_id'] . $url . '&page='. ($page + 1), true), 'next');
			}

			$data['params'] = $url_params->getParams();

			$data['continue'] = $this->url->link('common/home');

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => $this->language->get('text_home'),
				'href' => $this->url->link('common/home')
			);

			$data['breadcrumbs'][] = array(
				'text' => $this->language->get('Vendors'),
				'href' => $this->url->link('extension/ka_extensions/ka_multivendor/vendors')
			);

			$this->load->language('extension/ka_extensions/ka_multivendor/vendor');
			
			$this->showPage('extension/ka_extensions/ka_multivendor/vendor', $data);
			
		} else {

			$data['breadcrumbs'][] = array(
				'text' => $this->language->get('text_error'),
				'href' => $this->url->link('extension/ka_extensions/ka_multivendor/vendor', '')
			);

			$this->document->setTitle($this->language->get('text_error'));

			$data['heading_title'] = $this->language->get('text_error');

			$data['text_error'] = $this->language->get('text_error');

			$data['continue'] = $this->url->link('common/home');

			$this->response->addHeader($this->request->server['SERVER_PROTOCOL'] . ' 404 Not Found');

			$data['header'] = $this->load->controller('common/header');
			$data['footer'] = $this->load->controller('common/footer');
			$data['column_left'] = $this->load->controller('common/column_left');
			$data['column_right'] = $this->load->controller('common/column_right');
			$data['content_top'] = $this->load->controller('common/content_top');
			$data['content_bottom'] = $this->load->controller('common/content_bottom');

			$this->response->setOutput($this->load->view('error/not_found', $data));
		}
	}
}
class_alias(__NAMESPACE__ . '\ControllerVendor', 'ControllerExtensionKaExtensionsKaMultivendorVendor');