<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 143 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerVendorOrder extends \extension\ka_extensions\Controller {

	use \extension\ka_extensions\TraitControllerForm;

	protected $kamodel_vendor_order;
	protected $kamodel_vendor_order_histories;
	
	protected function onLoad() {
		$this->load->language('account/order');	
		$this->load->language('extension/ka_extensions/ka_multivendor/vendor/order');
	
		$this->load->model('localisation/language');
		$this->load->model('tool/upload');
		
		$this->kamodel_vendor_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/order');
		$this->kamodel_vendor_order_histories = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/order_histories');
		
		// check vendor access
		//
		if (!$this->customer->isVendor()) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
	}

  	public function index() {
  		return $this->getForm();
  	}  	
	
  	protected function getForm() {
    
  		$vendor_id = $this->customer->getId();
  	
  		$main_order_id = 0;
  		if (!empty($this->request->get['order_id'])) {
  			$main_order_id = $this->request->get['order_id'];
  		}
		
		$languages = $this->model_localisation_language->getLanguages();
		
		// initialize data array
		//
		$vendor_order = $this->kamodel_vendor_order->getVendorOrderByMainOrder($main_order_id, $vendor_id);
		if (empty($vendor_order)) {
			$this->addTopMessage("The order was not found", 'E');
			$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/vendor/orders', '', true));
		}
		$vendor_order_id = $vendor_order['vendor_order_id'];
				
		$order = $this->kamodel_vendor_order->getVendorOrder($vendor_order_id);
		
		$this->data['order'] = $order;
		
		$products = $this->kamodel_vendor_order->getVendorOrderProducts($vendor_order_id);
		
		$this->data['products'] = array();
		foreach ($products as $p) {
			$rec = $p;

			$rec['href']  = $this->url->link('product/product', 'product_id=' . $p['product_id'], true);
			
			$rec['price'] = $this->currency->format($p['price'] + $p['tax'], $order['currency_code'], $order['currency_value']);
			$rec['total'] = $this->currency->format($p['total'] + $p['tax'] * $p['quantity'], $order['currency_code'], $order['currency_value']);
			
			$this->data['products'][] = $rec;
		}
		
		$this->data['order']['date_added'] = date($this->language->get('datetime_format'), strtotime($order['date_added']));

		$this->data['order']['payment_address'] = $this->kamodel_vendor_order->getFormattedAddress($order, 'payment', $order['payment_address_format']);
		$this->data['order']['shipping_address'] = $this->kamodel_vendor_order->getFormattedAddress($order, 'shipping', $order['shipping_address_format']);

		// get order totals
		//
		$totals = $this->kamodel_vendor_order->getOrderTotals($vendor_order_id);
		$this->data['totals'] = array();
		foreach ($totals as $t) {
			$t['text'] = $this->currency->format($t['value'], $order['currency_code'], $order['currency_value']);
			$this->data['totals'][] = $t;
		}		
		
		// order fields
		//
		$fields = $this->kamodel_vendor_order->getOrderFields();
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['fields'] = $this->getFieldsWithData($fields, $order, $this->request->post, $this->errors);
		} else {
			$this->data['fields'] = $this->getFieldsWithData($fields, $order);
		}		

		$this->data['currency_unit'] = $this->currency->getSymbolLeft($this->config->get('config_currency'));
		$this->data['weight_unit'] = $this->weight->getUnit($this->config->get('config_weight_class_id'));

		// get custom shipping methods
		// note: it is possible to extract account_custom_fields and payment_custom_fields, but we don't do it for now
		//
		$this->data['shipping_custom_fields'] = $this->extractCustomFields($order['shipping_custom_field'], 'address');

		// this line will enable showing the account fields to a vendor
		// $this->data['account_custom_fields'] = $this->extractCustomFields($order['custom_field'], 'account');

		$this->data['ip']              = $order['ip'];
		$this->data['forwarded_ip']    = $order['forwarded_ip'];
		$this->data['user_agent']      = $order['user_agent'];
		$this->data['accept_language'] = $order['accept_language'];

		// order statuses
		$this->data['order_statuses'] = $this->kamodel_order_status->getVendorOrderStatuses();

		$this->data['is_status_editable'] = false;		
		if (!empty($order['delivery_status_id'])) {
			if ($this->kamodel_order_status->isVendorStatus($order['delivery_status_id'])) {
				$this->data['is_status_editable'] = true;
			}
		} else if ($this->kamodel_order_status->isVendorStatus($order['order_status_id'])) {
			$this->data['is_status_editable'] = true;
		}		
		
		$this->data['show_payment_address']   = false;
		$this->data['show_customer_env_info'] = false;
		
		// init general form variables
		//
  		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);
  		
   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('Account'),
			'href'      => $this->url->link('account/account', '', true),
   		);

   		$url = '';
   		
   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('Order'),
			'href'      => $this->url->link('extension/ka_extensions/ka_multivendor/vendor/orders', $url, true),
   		);

   		if (!empty($vendor_order_id)) {
   			$url = "&vendor_order_id=" . $vendor_order_id;
   		}
   		
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/order/save', $url, true);

		$this->data['cancel'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/order', $url, true);

		$this->data['languages'] = $this->model_localisation_language->getLanguages();

		$this->showPage('extension/ka_extensions/ka_multivendor/vendor/order_info');
  	}
  	
  	
	public function history() {
	
		$per_page = 10;

		if (isset($this->request->get['page'])) {
			$page = $this->request->get['page'];
		} else {
			$page = 1;
		}

		$data['histories'] = array();

		$results = $this->kamodel_vendor_order_histories->getOrderHistories($this->request->get['vendor_order_id'], ($page - 1) * $per_page, $per_page);

		foreach ($results as $result) {
			$data['histories'][] = array(
				'notify'     => $result['notify'] ? $this->language->get('text_yes') : $this->language->get('text_no'),
				'status'     => $result['status'],
				'comment'    => nl2br($result['comment']),
				'date_added' => date($this->language->get('date_format_short'), strtotime($result['date_added']))
			);
		}

		$history_total = $this->kamodel_vendor_order_histories->getTotalOrderHistories($this->request->get['vendor_order_id']);

		$pagination = new \KaPagination();
		$pagination->total = $history_total;
		$pagination->page = $page;
		$pagination->limit = $per_page;
		$pagination->url = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/order/history', 'vendor_order_id=' . $this->request->get['vendor_order_id'] . '&page={page}', true);

		$data['pagination'] = $pagination->render();

		$data['results'] = $pagination->getResults($this->language->get('text_pagination'));

		$this->response->setOutput($this->load->view('extension/ka_extensions/ka_multivendor/vendor/order/history', $data));
	}
	
	
	/*
		Ajax call from the order info page
	*/
	public function updateStatus() {
	
		$json = array();
	
		// Add keys for missing post vars
		$keys = array(
			'order_status_id',
			'comment'
		);

		foreach ($keys as $key) {
			if (!isset($this->request->post[$key])) {
				$this->request->post[$key] = '';
			}
		}

		$vendor_order_id = (int)$this->request->get['vendor_order_id'];
		
		$this->load->model('checkout/order');
	
		try {
			$this->kamodel_vendor_order->changeVendorStatus($vendor_order_id, 
				$this->request->post['order_status_id'], 
				$this->request->post['comment'],
				true
			);
		} catch (\Exception $e) {
			$json['error'] = $e->getMessage();
		}
		
		if (empty($json['error'])) {
			$json['success'] = $this->language->get('text_success');
		}
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
	
	
	protected function extractCustomFields($order_fields, $location) {

		$filter_data = array(
			'sort'  => 'cf.sort_order',
			'order' => 'ASC'
		);
		
		$return = array();

		$custom_fields = $this->kamodel_custom_field->getCustomFields($filter_data);
	
		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] != $location || !isset($order_fields[$custom_field['custom_field_id']])) {
				continue;
			}
			
			
			if ($custom_field['type'] == 'select' || $custom_field['type'] == 'radio') {
				$custom_field_value_info = $this->kamodel_custom_field->getCustomFieldValue($order_fields[$custom_field['custom_field_id']]);

				if ($custom_field_value_info) {
					$return[] = array(
						'name'  => $custom_field['name'],
						'value' => $custom_field_value_info['name'],
						'sort_order' => $custom_field['sort_order']
					);
				}
			}

			if ($custom_field['type'] == 'checkbox' && is_array($order_fields[$custom_field['custom_field_id']])) {
				foreach ($order_fields[$custom_field['custom_field_id']] as $custom_field_value_id) {
					$custom_field_value_info = $this->kamodel_custom_field->getCustomFieldValue($custom_field_value_id);

					if ($custom_field_value_info) {
						$return[] = array(
							'name'  => $custom_field['name'],
							'value' => $custom_field_value_info['name'],
							'sort_order' => $custom_field['sort_order']
						);
					}
				}
			}

			if ($custom_field['type'] == 'text' || $custom_field['type'] == 'textarea' || $custom_field['type'] == 'file' || $custom_field['type'] == 'date' || $custom_field['type'] == 'datetime' || $custom_field['type'] == 'time') {
				$return[] = array(
					'name'  => $custom_field['name'],
					'value' => $order_fields[$custom_field['custom_field_id']],
					'sort_order' => $custom_field['sort_order']
				);
			}

			if ($custom_field['type'] == 'file') {
				$upload_info = $this->model_tool_upload->getUploadByCode($order_fields[$custom_field['custom_field_id']]);

				if ($upload_info) {
					$return[] = array(
						'name'  => $custom_field['name'],
						'value' => $upload_info['name'],
						'sort_order' => $custom_field['sort_order']
					);
				}
			}
		}
		
		return $return;
	}
}

class_alias(__NAMESPACE__ . '\ControllerVendorOrder', 'ControllerExtensionKaExtensionsKaMultivendorVendorOrder');