<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 143 $)
	
*/

namespace extension\ka_extensions\ka_multivendor;

use extension\ka_extensions\UrlParams;

class ControllerVendorOrders extends \extension\ka_extensions\Controller { 

	protected $errors = array();
	protected $post = array();

	protected $kamodel_vendor_order;
	
	function onload() {
		// load langauge files
		$this->load->language('account/account');
		$this->load->language('extension/ka_extensions/ka_multivendor/common');

		// check vendor access
		//
		if (!$this->customer->isVendor()) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
		
		$this->kamodel_vendor_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/order');
	}
	
	
  	public function index() {
    	$this->getList();
  	}


  	protected function getList() {

  		$this->document->setTitle($this->language->get('text_orders_page_title'));
  	
  		$params = array(
			'sort'  => 'order_id',
			'sort'  => 'order_status_id',			
  			'order' => 'asc', 
  			'filter_customer'    => '',
  			'filter_placed_from' => '',
  			'filter_placed_to'   => '',
  			'limit' => $this->config->get('config_limit_admin'),
  			'page'  => 1,
  		);
  		
		$url_params = new UrlParams($this->request, $params);
		$params = $url_params->getParams();
		
		$this->data['sort_order_id'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/orders', '&' . $url_params->getUrlSort('order_id'), true);
		$this->data['sort_order_status_id'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/orders', '&' . $url_params->getUrlSort('order_status_id'), true);
		
		$this->session->data['orders_url'] = $url_params->getUrl();
		
  		$this->data['breadcrumbs'] = array();
  		
		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);

   		$this->data['breadcrumbs'][] = array(
       		'text' => $this->language->get('text_orders_page_title'),
   		);
		
		$url = $url_params->getUrl();
		if (!empty($url)) {
			$url = '&' . $url;
		}

		$vendor = $this->kamodel_vendor->getVendor();
		$this->data['store_commission'] = $vendor['store_commission'] . '%';
		
		$this->load->model('localisation/order_status');
		
		$this->data['orders'] = array();

		$filter = array(
  			'filter_customer'    => $params['filter_customer'],
  			'filter_placed_from' => $params['filter_placed_from'],
  			'filter_placed_to'   => $params['filter_placed_to'],
			'sort'  => $params['sort'],
  			'order' => $params['order'],
			'start' => ($params['page'] - 1) * $params['limit'],
			'limit' => $params['limit'],
		);			

		$orders_total = $this->kamodel_vendor_order->getOrdersTotal($filter);
		
		$results = $this->kamodel_vendor_order->getOrders($filter);
 
    	foreach ($results as $result) {

    		$order = $result;
    	
			$actions = array();

			$order['total'] = $this->currency->format($result['total'], $this->config->get('config_currency'));
			
			$order['order_status']    = $this->model_localisation_order_status->getOrderStatus($result['order_status_id']);
			$order['delivery_status'] = $this->model_localisation_order_status->getOrderStatus($result['delivery_status_id']);

			$order['display_commission_total'] = $this->currency->format($result['commission_total'], $this->config->get('config_currency'));
			$order['store_commission'] = $result['store_commission'] . '%';

			$order['date_added'] = date($this->language->get('date_format_short'), strtotime($result['date_added']));
			
			$actions[] = array(
				'text' => $this->language->get('View'),
				'href' => $this->url->link('extension/ka_extensions/ka_multivendor/vendor/order', '&order_id=' . $result['main_order_id'] . $url, true),
				'icon' => 'fa-eye',
			);

			$order['actions'] = $actions;
			
			$this->data['orders'][] = $order;
		}

		$currency = $this->config->get('config_currency');
		$this->data['currency_sign'] = $this->currency->getSymbolLeft($currency) . $this->currency->getSymbolRight($currency);
		
		$this->data['text_no_results']  = $this->language->get('text_no_results');
		$pagination = new \KaPagination();
		$pagination->total = $orders_total;
		$pagination->page  = $params['page'];
		$pagination->limit = $params['limit'];
		$pagination->text  = $this->language->get('text_pagination');

		$url = '&' . $url_params->getUrl(['page' => null]);
		$pagination->url = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/orders', $url . '&page={page}', true);

		$this->data['pagination'] = $pagination->render();
		$this->data['results'] = $pagination->getResults();
		
		$this->data['params'] = $url_params->getParams();
		
		$this->showPage('extension/ka_extensions/ka_multivendor/vendor/order_list');
  	}
}

class_alias(__NAMESPACE__ . '\ControllerVendorOrders', '\ControllerExtensionKaExtensionsKaMultivendorVendorOrders');