<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 154 $)
	
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerVendorProduct extends \extension\ka_extensions\Controller {

	use \extension\ka_extensions\TraitControllerForm {
		getFieldWithData as protected defaultGetFieldWithData;
	}

	protected $kamodel_vendor_product;
	protected $kamodel_vendor_category;
	
	function onload() {
		// load langauge files
		$this->load->language('account/account');
		$this->load->language('extension/ka_extensions/ka_multivendor/common');
		$this->load->language('extension/ka_extensions/ka_multivendor/vendor/product');

		// load a model file
		$this->kamodel_vendor_product  = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/product');
		$this->kamodel_vendor_category = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/category');
		$this->load->model('tool/image');
		
		// check vendor access
		//
		if (!$this->customer->isVendor()) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
	}

	
  	public function index() {
  	
		if (empty($this->request->get['product_id'])) {
			$vendor = $this->kamodel_vendor->getVendor();
			$products_total = $this->kamodel_vendor_product->getProductsTotal();
			if ($products_total >= $vendor['product_limit']) {
				$this->addTopMessage($this->language->get("You cannot add more products. The product limit reached"), 'E');
				$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/vendor/products', '', true));
				return;
			}
		}
  	
    	$this->getForm();
  	}

  	
  	public function save() {
  	
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {

			if (!empty($this->request->post['unlimited'])) {
				$this->request->post['quantity'] = 9999;
			}

		 	if ($this->validateForm()) {

		 		$error      = '';
		 		$product_id = 0;
		 		
	      		$product_id = $this->kamodel_vendor_product->saveProduct($this->request->post);
	      		if (empty($product_id)) {
		      		$error = $this->kamodel_vendor_product->getLastError();
		      	}
		  	
				$url = '';
				if (!empty($this->session->data['product_url'])) {
					$url = '&' . $this->session->data['product_url'];
				}
				
				if (empty($error)) {
					$this->addTopMessage($this->language->get('txt_saved_successfully'));
	    	  		$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/vendor/products', $url, true));
	    	  	} else {
		      		$this->addTopMessage($this->language->get('text_operation_failed') . ': ' . $error, 'E');
	    	  	}
    	  	}
		}
	
    	$this->getForm();
  	}
  	  	

  	protected function getForm() {

		$url = '';
		if (!empty($this->session->data['product_url'])) {
			$url = '&' . $this->session->data['product_url'];
		}

    	$this->document->setTitle($this->language->get('text_form_page_title'));
		
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();
		
		$product_id = 0;
		if (!empty($this->request->get['product_id'])) {
			$product_id = (int)$this->request->get['product_id'];
		}
		
		// get initial product data
		//
		$product             = array();
		$product_description = array();
		$product_images      = array();
		$product_attributes  = array();
		
		if (!empty($product_id)) {
			$product = $this->kamodel_vendor_product->getProduct($product_id);
			if (empty($product)) {
				$this->addTopMessage($this->language->get('You are not allowed to edit this product.'), 'E');
    	  		$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/vendor/products', $url, true));
			}
			$product_description = $this->kamodel_vendor_product->getProductDescription($product_id);
			$product_attributes  = $this->kamodel_vendor_product->getProductAttributes($product_id);
			$url .= '&product_id=' . $product_id;
		}
		
		// get fields for the product table
		//
		$fields = $this->kamodel_vendor_product->getProductFields();		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['fields'] = $this->getFieldsWithData($fields, $product, $this->request->post, $this->errors);
		} else {
			$this->data['fields'] = $this->getFieldsWithData($fields, $product);
		}	
		
		// get fields for the product description
		//
		$this->data['pd_fields'] = array();
		$fields = $this->kamodel_vendor_product->getProductDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['pd_fields'] = $this->getLanguageFieldsWithData('product_description', $fields, $product_description, $this->request->post, $this->errors);
		} else {
			$this->data['pd_fields'] = $this->getLanguageFieldsWithData('product_description', $fields, $product_description);
		}
		
		
		// get image list
		//
		$this->data['images_list'] = $this->getProductImagesList($product_id);
		$this->data['image_placeholder'] = $this->model_tool_image->resize('no_image.png', 100, 100);
		
		// get attributes
		//	
		if (!empty($this->data['fields']['category_id']['value'])) {
			$filter = array(
				'category_id' => $this->data['fields']['category_id']['value']
			);
			$fields = $this->kamodel_vendor_category->getCategoryAttributeFields($filter);
			$fields = $this->copyByLanguages($fields);
			
			if ($this->request->server['REQUEST_METHOD'] == 'POST') {
				$this->data['pa_fields'] = $this->getLanguageFieldsWithData('product_attributes', $fields, $product_attributes, $this->request->post, $this->errors);
			} else {
				$this->data['pa_fields'] = $this->getLanguageFieldsWithData('product_attributes', $fields, $product_attributes);
			}
		}

		$this->data['placeholder'] = $this->model_tool_image->resize('no_image.png', 100, 100);
		
		// define breadcrumbs
		//
  		$this->data['breadcrumbs'] = array();

   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('text_home'),
			'href'      => $this->url->link('common/home', '', true), 
   		);

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', true)
		);
   		
   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('text_list_page_title'),
			'href'      => $this->url->link('extension/ka_extensions/ka_multivendor/vendor/products', $url, true),
   		);


   		$text = $this->language->get('New Product');
   		if (!empty($this->data['fields']['model']['value'])) {
   			$text = $this->data['fields']['model']['value'];
   		}
   		
   		$this->data['breadcrumbs'][] = array(
       		'text' => $text,
   		);   		
   		
   		$this->data['languages'] = $languages;
   		
   		// define action links
   		//
   		if (!empty($product_id)) {
			$this->data['copy']   = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/product/copy', '&product_id=' . $product_id, true);
		}
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/product/save', $url, true);
		$this->data['cancel'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/products', $url, true);

		$this->showPage('extension/ka_extensions/ka_multivendor/vendor/product_form');
  	}

 	
	protected function validateForm() {

		// check product fields
		//
		$has_errors = false;
		
		$fields = $this->kamodel_vendor_product->getProductFields();
		if (!$this->validateFields($fields, $this->request->post)) {
			$has_errors = true;
		}
		
		if (!empty($this->request->post['special_price'])) {
			if ($this->request->post['special_price'] > $this->request->post['price']) {
				$this->request->post['special_price'] = '';
			}
		}

		// check product description fields
		//
		$fields = $this->kamodel_vendor_product->getProductDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		if (!$this->validateLanguageFields('product_description', $fields, $this->request->post)) {
			$has_errors = true;
		}
	
      	if ($has_errors) {
      		$this->addTopMessage($this->language->get("text_invalid_form_data"), 'E');
      		return false;
      	}

    	return true;		
	}

	
	protected function getFieldWithData($k, $f, $old_data, $new_data = null) {
	
		if ($k == 'category_id') {
		
			$f['input_code'] = 'category';

			if (!empty($f['value'])) {
				$category = $this->kamodel_vendor_category->getCategory($f['value']);
				if (defined('MV_FULL_CATEGORY_PATH')) {
					$f['input_value'] = ($category['path']) ? $category['path'] . ' &gt; ' . $category['name'] : $category['name'];
				} else {
					$f['input_value'] = $category['name'];
				}
			}
		}
		
		$f = $this->defaultGetFieldWithData($k, $f, $old_data, $new_data);

		if (!empty($f['format'])) {
			if ($f['format'] == 'weight') {
				$val = (float)$f['value'];
				$f['value'] = $this->weight->format($val, 0);
			}
			if ($f['format'] == 'length') {
				$val = (float)$f['value'];
				$f['value'] = $this->length->format($val, 0);
			}
		}
		
		return $f;
	}

	
	protected function getProductImagesList($product_id) {
	
		$ret_images = array();
		
		$fields = $this->kamodel_vendor_product->getProductImageFields();

		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$images = $this->request->post['product_images'];
		} else {
			$images = $this->kamodel_vendor_product->getProductImages($product_id);
		}
		
		if (!empty($images)) {
			foreach ($images as &$image) {

				if (empty($image['image'])) {
					continue;
				}
				
				if (is_file(DIR_IMAGE . $image['image'])) {
					$image['thumb'] = $this->model_tool_image->resize($image['image'], 100, 100);
				} else {
					$image['thumb'] = $this->model_tool_image->resize('no_image.png', 100, 100);
				}
			
				$ret_images[] = $image;
			}
		}
		
		$empty_image = array(
			'image'      => $fields['image']['default_value'],
			'thumb'      => $this->model_tool_image->resize('no_image.png', 100, 100),
			'sort_order' => count($images) * 10,
		);
		$ret_images[] = $empty_image;

		array_unshift($ret_images, $empty_image);

		$field = array(
			'code'   => 'product_image',
			'title'  => $this->language->get('Images'),
			'images' => $ret_images,
		);
		
		return $field;
	}
	

	public function autocompleteCategory() {
		$json = array();

		if (isset($this->request->get['filter_name'])) {

			$filter_data = array(
				'filter_name' => $this->request->get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$results = $this->kamodel_vendor_category->getCategories($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'category_id' => $result['category_id'],
					'name'        => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}

class_alias(__NAMESPACE__ . '\ControllerVendorProduct', '\ControllerExtensionKaExtensionsKaMultivendorVendorProduct');