<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 140 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ControllerVendorShipping extends \extension\ka_extensions\Controller {
	use \extension\ka_extensions\TraitControllerForm;

	protected $kamodel_shipping;
	protected $kamodel_geo_zone;
	
	protected function onLoad() {
		$this->load->language('extension/ka_extensions/ka_multivendor/common');
		$this->load->language('extension/ka_extensions/ka_multivendor/vendor/shipping');
		$this->load->model('localisation/language');
		
		$this->kamodel_shipping = $this->load->kamodel('extension/ka_extensions/ka_multivendor/shipping');
		
		// check vendor access
		//
		if (!$this->customer->isVendor()) {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
		
		if ($this->config->get('ka_multivendor_shipping_setup') != 'custom') {
			$this->response->redirect($this->url->link('account/account', '', true));
		}
	}

  	public function index() {
  		return $this->getForm();
  	}

  	public function save() {

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validateForm()) {
		
			$shipping_method_id = 0;
			$data = $this->request->post;
			if (!empty($this->request->get['shipping_method_id'])) {
				$shipping_method_id = (int)$this->request->get['shipping_method_id'];
			}
			$data['shipping_method_id'] = $shipping_method_id;
		
			try {
	      		$ok = $this->kamodel_shipping->saveShipping($data);
	      	} catch (\Exception $e) {
	      		$ok = false;
	      	}

			$url = '';
			if (isset($this->session->data['smethods_url'])) {
				$url = $this->session->data['smethods_url'];
			}
			
			//!! тут надо разбирать url и добавлять shipping_method_id только в url где его не было ранее
			// тоже самое для продукта сделать
			if (!empty($shipping_method_id)) {
				$url .= '&shipping_method_id=' . $shipping_method_id;
			}

			if ($ok) {
				$this->addTopMessage($this->language->get('txt_saved_successfully'));
    	  		$this->response->redirect($this->url->link('extension/ka_extensions/ka_multivendor/vendor/shippings', $url, true));
    	  	} else {
	      		$this->addTopMessage($this->language->get('text_operation_failed'), 'E');
    	  	}
		}
	
    	$this->getForm();
  	}

  	
	protected function validateForm() {

		$has_errors = false;

		// check shipping description fields
		//
		$fields = $this->kamodel_shipping->getShippingDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		if (!$this->validateLanguageFields('shipping_description', $fields, $this->request->post)) {
			$has_errors = true;
		}
	
      	if ($has_errors) {
      		$this->addTopMessage($this->language->get("text_invalid_form_data"), 'E');
      		return false;
      	}
      	
      	return true;
	}
	
	
  	protected function getForm() {
    
  		$shipping_method_id = 0;
  		if (!empty($this->request->get['shipping_method_id'])) {
  			$shipping_method_id = $this->request->get['shipping_method_id'];
  		}  		
  	
		$url = '';
		if (isset($this->session->data['smethods_url'])) {
			$url = $this->session->data['smethods_url'];
		}
		
		$languages = $this->model_localisation_language->getLanguages();
		
		// initialize data array
		//
		$shipping            = array();
		$shipping_description = array();

		if (!empty($shipping_method_id)) {
			$shipping = $this->kamodel_shipping->getShipping($shipping_method_id);
			$shipping_description = $this->kamodel_shipping->getShippingDescription($shipping_method_id);
		}
		
		// shipping fields
		//
		$fields = $this->kamodel_shipping->getShippingFields();
		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['fields'] = $this->getFieldsWithData($fields, $shipping, $this->request->post, $this->errors);
		} else {
			$this->data['fields'] = $this->getFieldsWithData($fields, $shipping);
		}

		// shipping description fields		
		//
		$fields = $this->kamodel_shipping->getShippingDescriptionFields();
		$fields = $this->copyByLanguages($fields);
		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$this->data['sd_fields'] = $this->getLanguageFieldsWithData('shipping_description', $fields, $shipping_description, $this->request->post, $this->errors);
		} else {
			$this->data['sd_fields'] = $this->getLanguageFieldsWithData('shipping_description', $fields, $shipping_description);
		}

		// shipping rates
		//
		$this->kamodel_geo_zone = $this->load->kamodel('extension/ka_extensions/ka_multivendor/geo_zone');
		$geo_zones = $this->kamodel_geo_zone->getGeoZones();
		
		$all_zone = array(
			'geo_zone_id' => 0,
			'name'        => $this->language->get('All Zones')
		);
		array_unshift($geo_zones, $all_zone);
		
		$rates_by_zones = array();		
		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			if (!empty($this->request->post['rates_by_zones'])) {
				$rates_by_zones = $this->request->post['rates_by_zones'];
			}
		} elseif (!empty($shipping_method_id)) {
			$rates_by_zones = $this->kamodel_shipping->getRatesByZones($shipping_method_id);
		}
		
		// add a sample row for js cloning operation
		//
		foreach ($geo_zones as $geo_zone) {
			$sample_rate = array(
				'weight_min'  => 0,
				'weight_max'  => 0,
				'subtotal_min' => 0,
				'subtotal_max' => 0,
				'flat_charge' => 0,
				'per_item_charge' => 0,
				'per_weight_charge' => 0,
				'subtotal_percentage' => 0,
			);
			if (!empty($rates_by_zones[$geo_zone['geo_zone_id']])) {
				array_unshift($rates_by_zones[$geo_zone['geo_zone_id']], $sample_rate);
			} else {
				$rates_by_zones[$geo_zone['geo_zone_id']][] = $sample_rate;
			}
		}
		
		$this->data['rates_by_zones'] = $rates_by_zones;

		$this->data['geo_zones'] = $geo_zones;
		
		$this->data['currency_unit'] = $this->currency->getSymbolLeft($this->config->get('config_currency'));
		$this->data['weight_unit'] = $this->weight->getUnit($this->config->get('config_weight_class_id'));
				
		// init general form variables
		//
  		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);
  		
   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('Account'),
			'href'      => $this->url->link('account/account', '', true),
   		);

   		$this->data['breadcrumbs'][] = array(
       		'text'      => $this->language->get('Shipping'),
			'href'      => $this->url->link('extension/ka_extensions/ka_multivendor/vendor/shippings', $url, true),
   		);

   		if (!empty($shipping_method_id)) {
   			$url = "&shipping_method_id=" . $shipping_method_id;
   		}
   		
		$this->data['action'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/shipping/save', $url, true);

		$this->data['cancel'] = $this->url->link('extension/ka_extensions/ka_multivendor/vendor/shippings', $url, true);

		$this->data['languages'] = $this->model_localisation_language->getLanguages();

		$this->showPage('extension/ka_extensions/ka_multivendor/vendor/shipping_form');
  	}
}

class_alias(__NAMESPACE__ . '\ControllerVendorShipping', 'ControllerExtensionKaExtensionsKaMultivendorVendorShipping');