<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 152 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ModelCheckoutOrder extends \ModelCheckoutOrder {

	protected $tbl_ka_vendor_order;

	protected function isMultivendorData($data) {

		if (empty($data['products'])) {
			return false;
		}
	
		foreach ($data['products'] as $product) {
			if (!empty($product['vendor_id'])) {
				return true;
			}
		}

		return false;
	}
	
	
	protected function setVendorToOrderItems($order_id, $data) {
	
		if (empty($data['products'])) {
			return false;
		}
	
		foreach ($data['products'] as $product) {
			$this->db->query("UPDATE " . DB_PREFIX . "order_product SET
				vendor_id = '" . (int)$product['vendor_id'] . "'
				WHERE 
					product_id = '" . (int)$product['product_id'] . "'
					AND order_id = '" . (int)$order_id . "'
			");
		}
	}

	
	public function addOrder($data) {

		if (!empty($this->session->data['shipping_method_by_vendor'])) {
			$data['shipping_method_by_vendor'] = $this->session->data['shipping_method_by_vendor'];
		}
	
		$order_id = parent::addOrder($data);
		
		if (!$this->isMultivendorData($data)) {
			return $order_id;
		}
		
		$this->setVendorToOrderItems($order_id, $data);
		
		$this->saveVendorOrders($order_id, $data);
		
		return $order_id;
	}


	public function editOrder($order_id, $data) {

		if (!empty($this->session->data['shipping_method_by_vendor'])) {
			$data['shipping_method_by_vendor'] = $this->session->data['shipping_method_by_vendor'];
		}
	
		parent::editOrder($order_id, $data);
	
		if (empty($data['is_multivendor'])) {
			return;
		}
		
		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		$vorders = $kamodel_order->getVendorOrdersByMainOrder($order_id);

		if (!empty($data['shipping_method_by_vendor'])) {
			foreach ($data['shipping_method_by_vendor'] as $vendor_id => &$sm) {
				if (!empty($sm['code']) && $sm['code'] == 'original.original') {
					$sm['code']  = $vorders[$vendor_id]['shipping_code'];
					$sm['title'] = $vorders[$vendor_id]['shipping_method'];
					$sm['cost']  = $vorders[$vendor_id]['shipping_cost'];
				}
			}
		}

		$this->setVendorToOrderItems($order_id, $data);
		
		$this->saveVendorOrders($order_id, $data);
	}
	

	public function deleteOrder($order_id) {
	
		$this->db->query("DELETE FROM `" . DB_PREFIX . "ka_vendor_order` WHERE main_order_id = '" . (int)$order_id . "'");
	
		parent::deleteOrder($order_id);
	}


	protected function getDataByVendor($data) {
	
		$vendors = array();

		// split data by vendors
		//
		foreach ($data['products'] as $p) {
			$vendors[$p['vendor_id']]['products'][] = $p;
		}
		
		// collect vendor data
		//
		foreach ($vendors as &$v) {
			
			$v['subtotal'] = 0;
		
			foreach ($v['products'] as $p) {
				$v['subtotal'] += $p['total'];
			}
		}
		
		return $vendors;
	}
	
	
	protected function saveVendorOrders($order_id, $data) {

		$this->db->query("DELETE FROM " . DB_PREFIX . "ka_vendor_order WHERE main_order_id = '" . (int)$order_id . "'");
	
		$tbl_ka_vendor_order = new table\KaVendorOrder($this->registry);
		
		$kamodel_vendor = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor');
		
		$data_by_vendors = $this->getDataByVendor($data);
		
		foreach ($data_by_vendors as $vendor_id => $v) {
		
			$store_commission = 0;
			$commission_total = 0;
			
			$shipping = array(
				'title' => '',
				'code'   => '',
				'cost'   => 0,
			);
			
			if (!empty($data['shipping_method_by_vendor'][$vendor_id])) {
				$shipping = $data['shipping_method_by_vendor'][$vendor_id];
			}

			$total = ($v['subtotal'] + $shipping['cost']);
			
			$vendor = $kamodel_vendor->getVendor($vendor_id);
			if (!empty($vendor)) {
				$store_commission = $vendor['store_commission'];
				$commission_total = round($total / 100 * $vendor['store_commission'], 2);
			}
		
			// add a new ka_vendor_order
			$vo = array(
				'main_order_id' => $order_id,
				'vendor_id'     => $vendor_id,
				'delivery_status_id' => 0,
				'shipping_method'    => $shipping['title'],
				'shipping_code'      => $shipping['code'],
				'shipping_cost'      => $shipping['cost'],
				'store_commission'   => $store_commission,
				'commission_total'   => $commission_total,
				'total'              => $total,
			);
			
			$tbl_ka_vendor_order->insert($vo);
		}
	}
	
	
	public function addOrderHistory($order_id, $order_status_id, $comment = '', $notify = false, $override = false) {

		$order_info = $this->getOrder($order_id);
		if (empty($order_info)) {
			return;
		}
	
		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		
		parent::addOrderHistory($order_id, $order_status_id, $comment, $notify, $override);
		
		$kamodel_order->onMainOrderStatusUpdate($order_info, $order_status_id);
	}
}