<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 120 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ModelExtensionTotalShipping extends \extension\total\shipping\ModelExtensionTotalShipping {

	/*
		We have to return an array with values:
		
		[total] - increase the total by the total shipping cost
		[totals] - add a new total to that array. It will contain these fields:
			array(
				'code'       => 'shipping',
				'title'      => <Shipping method name>
				'value'      => <total cost>
				'sort_order' => $this->config->get('total_shipping_sort_order')
			)
	*/
	public function getTotal($total) {

		if (!$this->cart->isMultivendorCart()) {
			return parent::getTotal($total);
		}
		
		if (empty($this->session->data['shipping_method_by_vendor'])) {
			return;
		}
		
		$vendors = $this->cart->getVendors();

		$shipping_total = 0;
		$cart_total = 0;
		
		foreach ($vendors as $vendor_id => $v) {

			if (empty($this->session->data['shipping_method_by_vendor'][$vendor_id])) {
				continue;
			}
			
			// here we calculate the shiping cost for a single vendor with a default routine
			// but we need to prepare some data for it
			//
			$totals = array();
			$taxes = $this->cart->getTaxes();
			$vendor_total = 0;
		
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$vendor_total
			);
			
			$this->session->data['shipping_method'] = $this->session->data['shipping_method_by_vendor'][$vendor_id];
			
			parent::getTotal($total_data);

			$cart_total += $vendor_total;
			
			// after getting the results we sum up the shipping cost with other shipping costs
			// the tax has to be taken into account as well
			//
			if (!empty($totals)) {
				foreach ($totals as $t) {
					if ($t['code'] == 'shipping') {
						$shipping_total += $total_data['total'];
						break;
					}
				}
			}
			
			if (!empty($total_data['taxes'])) {
				foreach ($total_data['taxes'] as $k => $v) {
					if (!isset($total['taxes'][$k])) {
						$total['taxes'][$k] = $v;
					} else {
						$total['taxes'][$k] += $v;
					}
				}
			}
		}
		
		$total['totals'][] = array(
			'code'       => 'shipping',
			'title'      => $this->language->get('Total shippnig'),
			'value'      => $shipping_total,
			'sort_order' => $this->config->get('total_shipping_sort_order')
		);

		$total['total'] += $cart_total;
	}
}