<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 150 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ModelShipping extends \extension\ka_extensions\Model {

	private $kamodel_geo_zone;

	public function isVendorShipping($shipping_method_id) {

		$shipping = $this->db->query("SELECT vendor_id FROM " . DB_PREFIX . "ka_shipping_methods
			WHERE shipping_method_id = '" . (int)$shipping_method_id . "'
		")->row;
		
		if (empty($shipping)) {
			return false;
		}
		
		if ($shipping['vendor_id'] != $this->customer->getId()) {
			return false;
		}
		
		return true;
	}
	
	
	public function saveShipping($data) {

		if (empty($data)) {
			return;
		}
		
		$data['vendor_id'] = $this->customer->getId();
	
		// create/update the shipping record
		//
		$shipping_method_id = 0;
		
		if (empty($data['shipping_method_id'])) {
		
			$total = $this->getShippingsTotal();
			if ($total >= 100) {
				$this->lastError = "The number of shipping methods for the vendor reached the limit (100)";
				return false;
			}
		
			$shipping_method_id = $this->tbl_ka_shipping_methods->insert($data);
		} else {
			$shipping_method_id = (int)$data['shipping_method_id'];	
			if (!$this->isVendorShipping($shipping_method_id)) {
				return false;
			}
			$this->tbl_ka_shipping_methods->update($data, "shipping_method_id = '$shipping_method_id'");
		}
		
		if (empty($shipping_method_id)) {
			return false;
		}

		// save shipping description
		//
		$this->tbl_ka_shipping_methods_description->setValue('shipping_method_id', $shipping_method_id);

		$this->load->model('localisation/language');		
		
		$languages = $this->model_localisation_language->getLanguages();
		
		foreach ($languages as $l) {
		
			if (!empty($data['shipping_description'][$l['language_id']])) {
				$rec = $data['shipping_description'][$l['language_id']];
			} else {
				$rec = array();
			}
			$rec['language_id'] = (int)$l['language_id'];
			
			$this->tbl_ka_shipping_methods_description->save($rec);
		}
		
		// save shipping store
		//
		$rec = array(
			'shipping_method_id' => $shipping_method_id,
			'store_id' => (int)$this->config->get('config_store_id')
		);
		$this->tbl_ka_shipping_methods_to_stores->save($rec);
		
		// create a record in the extension table
		//
		if (empty($data['status'])) {
			$this->db->query("DELETE FROM " . DB_PREFIX . "extension WHERE
				type = 'shipping'
				AND code = '" . 'ka_smethods_' . $shipping_method_id . "'
			");
		} else {
			$row = $this->db->query("SELECT * FROM " . DB_PREFIX . "extension WHERE
				type = 'shipping'
				AND code = '" . 'ka_smethods_' . $shipping_method_id . "'
			")->row;
			
			if (empty($row)) {
				$rec = array(
					'type' => 'shipping',
					'code' => 'ka_smethods_' . $shipping_method_id
				);
				$this->kadb->queryInsert("extension", $rec);
			}
		}
		
		// save rates by zones
		//
		$this->kamodel_geo_zone = $this->load->kamodel('extension/ka_extensions/ka_multivendor/geo_zone');
		$geo_zones = $this->kamodel_geo_zone->getGeoZones();
		array_unshift($geo_zones, array(
			'geo_zone_id' => 0,
		));
		foreach ($geo_zones as $gz) {
			$rates = !empty($data['rates_by_zones'][$gz['geo_zone_id']]) ? $data['rates_by_zones'][$gz['geo_zone_id']] : array();
			
			$this->db->query("START TRANSACTION");
			$this->db->query("DELETE FROM " . DB_PREFIX . "ka_shipping_rates WHERE
				shipping_method_id = '$shipping_method_id' AND geo_zone_id = '" . $gz['geo_zone_id'] . "'
			");

			$total = 0;
			foreach ($rates as $rate) {
				if (empty($rate)) {
					continue;
				}
				
				// do not save more than 500 rates per zone
				if ($total++ > 500) {
					continue;
				}
								
				$rate['shipping_method_id'] = $shipping_method_id;
				$rate['geo_zone_id']  = $gz['geo_zone_id'];
				$rate['service_code'] = '';
				$this->tbl_ka_shipping_rates->save($rate);
			}
			$this->db->query("COMMIT");
		}
		
		return $shipping_method_id;
	}

	
	public function deleteShipping($shipping_method_id) {

		// verify that the method belongs to the vendor
		$shipping = $this->getShipping($shipping_method_id);
		if (empty($shipping)) {
			return;
		}
	
		$this->db->query("DELETE FROM " . DB_PREFIX . "ka_shipping_methods WHERE shipping_method_id = '" . (int)$shipping_method_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "ka_shipping_methods_description WHERE shipping_method_id = '" . (int)$shipping_method_id . "'");
	}

		
	public function getShipping($shipping_method_id) {
	
		$shipping = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_shipping_methods 
			WHERE shipping_method_id = '" . (int)$shipping_method_id . "'
			AND vendor_id = '" . $this->customer->getId() . "'
		")->row;
		
		if (empty($shipping)) {
			return array();
		}
		
		return $shipping;
	}

	
	public function getShippingDescription($shipping_method_id) {
		$shipping = array();
		
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_shipping_methods_description WHERE shipping_method_id = '" . (int)$shipping_method_id . "'");
		
		foreach ($query->rows as $result) {
			$shipping[$result['language_id']] = $result;
		}
		
		return $shipping;
	}


	protected function getRecords($data = array()) {

		$language_id = (int)$this->config->get('config_language_id');

		$where = array();
		if (!empty($data['where'])) {
			$where[] = $data['where'];
		}

		$where[] = "vendor_id = '" . (int)$this->customer->getId() . "'";
					
		if (empty($data['fields'])) {
			$fields = 'ksm.*, ksmd.name, ksmd.descr, ksmd.language_id';
		} else {
			$fields = $data['fields'];
		}
		
		$sql = "SELECT $fields FROM " . DB_PREFIX . "ka_shipping_methods ksm";
		
		$sql .= " LEFT JOIN " . DB_PREFIX . "ka_shipping_methods_description ksmd
				ON (ksm.shipping_method_id = ksmd.shipping_method_id 
					AND ksmd.language_id = $language_id
				)";

		
		if (!empty($where)) {
			$sql .= " WHERE " . implode(" AND ", $where);
		}
		
		$sort_data = array(
			'ksm.name',
			'ksm.status',
		);
		
		if (!empty($data['sort']) && in_array($data['sort'], $sort_data)) {
		
			$sql .= " ORDER BY " . $data['sort'];
			
			if (!empty($data['order'])) {
				$sql .= ' ' . $data['order'];
			}
		}
		
		$query = $this->db->query($sql);
		
		return $query;
	}


	public function getShippings($data = array()) {
	
		$shippings = array();
	
		$qry = $this->getRecords($data);

		if (!empty($qry->rows)) {
			$shippings = $qry->rows;
		}
		
		return $shippings;
	}

	
	public function getShippingsTotal($data = array()) {
	
      	$data['fields'] = 'COUNT(*) AS total';
   	
      	$data['where']  = "ksmd.language_id = '" . $this->config->get('config_language_id') . "'";

		if (isset($data['limit'])) {
			unset($data['limit']);
		}
		if (isset($data['start'])) {
			unset($data['start']);
		}

		$qry = $this->getRecords($data);
		
		if (empty($qry->row)) {
			return 0;
		}

		return $qry->row['total'];
	}

	public function getShippingFields() {
	
		$fields = array(
			'shipping_method_id' => array(
				'code' => 'shipping_method_id',
			),
			'status' => array(
				'code' => 'status',
				'options' => array(
					'0' => $this->language->get('txt_disabled'),
					'1' => $this->language->get('txt_enabled'),
				)
			),
		);
		
		return $fields;
	}

	public function getShippingDescriptionFields() {
	
		$fields = array( 
			'name' => array(
				'code'     => 'name',
				'required' => true,
				'section' => 'description',				
			),
			'description' => array(
				'code'     => 'description',
				'section' => 'description',
			),
		);

		return $fields;
	}

	public function enableShipping($shipping_method_id) {
	
		$data = array(
			'shipping_method_id' => $shipping_method_id,
			'status' => 1
		);
	
		$this->tbl_ka_shipping_methods->save($data);
	}
	

	public function disableShipping($shipping_method_id) {
	
		$data = array(
			'shipping_method_id' => $shipping_method_id,
			'status' => 0
		);
		
		$this->tbl_ka_shipping_methods->save($data);		
	}
	
	
	public function getRates($method_id) {
		$qry = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_shipping_rates WHERE
			shipping_method_id = '" . (int)$method_id . "'"
		);
		
		return $qry->rows;
	}		
	
	
	public function getRatesByZones($method_id) {
		$rates = $this->getRates($method_id);
		
		if (empty($rates)) {
			return $rates;
		}
		
		$rates_by_zones = array();
		foreach ($rates as $rate) {
			$rates_by_zones[$rate['geo_zone_id']][] = $rate;
		}
		
		return $rates_by_zones;
	}
	
	/*
		$code - oc_extension.code, example: flat, ka_smethods_29.
	
		Returns
			0    - shipping is defined by admin
			null - vendor shipping is not available
			<N>  - vendor_id
	*/
	public function getShippingVendor($code) {
	
		if (substr($code, 0, 11) != 'ka_smethods') {
			return 0;
		}

		$ka_shipping_id = (int)substr($code, 12);

		$shipping = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_shipping_methods WHERE
			shipping_method_id = '$ka_shipping_id'
		")->row;

		if (empty($shipping)) {
			return null;
		}
		
		$vendor_id = $shipping['vendor_id'];
		
		return $vendor_id;
	}
	
}

class_alias(__NAMESPACE__ . '\ModelShipping', '\ModelExtensionKaExtensionsKaMultivendorShipping');