<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 118 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ModelVendor extends AModelVendor {

	protected function onLoad() {
	
		$this->load->model('account/customer');
		$this->load->model('account/custom_field');
	}


	/*
		Return internal vendor data
	*/
	public function getVendor($vendor_id = null) {
	
		if (is_null($vendor_id)) {
			$vendor_id = $this->customer->getId();
		}

		// get any vendor information
		//
		$vendor = $this->getVendorInternal($vendor_id);
		if (empty($vendor) || empty($vendor_id)) {
			// exit when the vendor is 'admin'/'not specified' (vendor_id = 0)
			return $vendor;
		}
		
		$customer_id = $vendor_id;
		
		// get customer profile for non empty vendor ids
		//
		$customer = $this->model_account_customer->getCustomer($customer_id);
		if (empty($customer)) {
			$no_vendor = array(
				'name'      => $this->language->get('Deleted Vendor'),
				'vendor_id' => $customer_id,
			);
			return $no_vendor;
		}

		$vendor = array_merge($customer, $vendor);
		
		return $vendor;
	}
	
	
	public function getVendorStatus($customer_id = null) {
		if (empty($customer_id)) {
			$customer_id = $this->customer->getId();
		}
		
		return parent::getVendorStatus($customer_id);
	}
	
	
	public function addVendor($data) {

		$customer_id = $this->customer->getId();
		if (empty($customer_id)) {
			if (!empty($data['custom_field'])) {
				$data['custom_field'] = array(
					'account' => $data['custom_field']
				);
			}
			$customer_id = $this->model_account_customer->addCustomer($data);
		}

		// add a vendor record
		//
		$rec = array(
			'customer_id'      => $customer_id,
			'product_limit'    => $this->config->get('ka_multivendor_default_products_limit'),
			'store_commission' => $this->config->get('ka_multivendor_default_commision'),
		);
		
		if (isset($data['vendor_custom_field'])) {
			$rec['vendor_custom_field'] = $data['vendor_custom_field'];
		}
		
		parent::addVendor($rec);
		
		// queue the vendor for review
		//
		$this->db->query("INSERT INTO `" . DB_PREFIX . "customer_approval` 
			SET customer_id = '" . (int)$customer_id . "', type = 'vendor', date_added = NOW()
		");
		
		// send an email to the admin about adding a new vendor account
		
		$this->load->model('account/customer');
		
		$customer = $this->model_account_customer->getCustomer($customer_id);
		
		$mail = new \KaMail($this->config->get('config_admin_language'));
		$mail->data['customer'] = $customer;
		$mail->sendContent($this->config->get('config_email'), 'A new vendor registration', 
			'extension/ka_extensions/ka_multivendor/mail/vendor_registered'
		);
		
		return $customer_id;
	}
	

	public function editVendor($customer_id, $data) {
	
		$customer_id = $this->customer->getId();
		if (empty($customer_id)) {
			return;
		}
		
		$rec = array();
		if (isset($data['vendor_custom_field'])) {
			$rec['vendor_custom_field'] = $data['vendor_custom_field'];
		}
		if (isset($data['vendor_description'])) {
			$rec['vendor_description'] = $data['vendor_description'];
		}

		if (isset($data['vendor_logo'])) {
			$rec['vendor_logo'] = $data['vendor_logo'];
		}
		
		parent::editVendor($customer_id, $rec);
		
		return $customer_id;
	}
	
	/*
		$form
			R  - register customer and vendor account at the same time. This form contains fields from customer
			     and vendor forms.
			V  - regier vendor account only
			E  - edit vendor account data
		
	*/
	public function getAccountFields($form = 'R') {
	
		$fields = array();
		
		if ($form == 'R') {
		
			$fields = array(
				'firstname' => array(
					'code'     => 'firstname',
					'required' => true,
					'section' => $this->language->get('Your Personal Details'),
					'sort_order' => 1,				
				),
				'lastname' => array(
					'code' => 'lastname',
					'required' => true,
					'sort_order' => 2,
				),
				'email' => array(
					'code' => 'email',
					'type' => 'email',
					'required' => true,
					'sort_order' => 3,				
				),
				'telephone' => array(
					'code' => 'telephone',
					'type' => 'tel',
					'required' => true,
					'sort_order' => 4,
				),
			);
		}

		$account_fields = array();
		$vendor_fields = array();
		
		// update customer custom fields
		//
		$customer_group_id = $this->config->get('config_customer_group_id');
		$custom_fields = $this->model_account_custom_field->getCustomFields($customer_group_id);
		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] != 'account') {
				continue;
			}
			
			$custom_field['code']  = 'cf' . $custom_field['custom_field_id'];
			$custom_field['title'] = $custom_field['name'];
			$custom_field['is_custom'] = true;
			
			if (!empty($custom_field['custom_field_value'])) {
				$options = array();
				foreach ($custom_field['custom_field_value'] as $k => $v) {
					$options[$v['custom_field_value_id']] = $v['name'];
				}
				$custom_field['options'] = $options;
			}
			
			$custom_field['name']  = 'custom_field[' . $custom_field['custom_field_id'] . ']';
			$account_fields[$custom_field['code']] = $custom_field;
		}
		
		// update vendor custom fields
		//
		$customer_group_id = $this->config->get('config_customer_group_id');
		$custom_fields = $this->model_account_custom_field->getVendorCustomFields($customer_group_id);
		foreach ($custom_fields as $custom_field) {
			$custom_field['code']  = 'cf' . $custom_field['custom_field_id'];
			$custom_field['title'] = $custom_field['name'];
			$custom_field['is_vendor_custom'] = true;
			
			if (!empty($custom_field['custom_field_value'])) {
				$options = array();
				foreach ($custom_field['custom_field_value'] as $k => $v) {
					$options[$v['custom_field_value_id']] = $v['name'];
				}
				$custom_field['options'] = $options;
			}
			
			$custom_field['name']  = 'vendor_custom_field[' . $custom_field['custom_field_id'] . ']';
			$vendor_fields[$custom_field['code']] = $custom_field;
		}

		// sort account fields
		//
		if ($form == 'R') {
			$fields = array_merge($fields, $account_fields);
			uasort($fields, array($this, "sortArraysBySortOrder"));
		}

		// sort vendor fields
		//
		if (!empty($vendor_fields)) {
			uasort($vendor_fields, array($this, "sortArraysBySortOrder"));
			
			reset($vendor_fields);
			$fkey = key($vendor_fields);
			$vendor_fields[$fkey]['section'] = $this->language->get('Your Vendor Details');
			
			$fields = array_merge($fields, $vendor_fields);
		}
		
		if ($form == 'R') {
			$fields = array_merge($fields, array(
				'password' => array(
					'code' => 'password',
					'type' => 'password',
					'required' => true,
					'section' => $this->language->get('text_your_password'),
				),
				'confirm' => array(
					'code' => 'confirm',
					'type' => 'password',
					'required' => true,
				),
				'newsletter' => array(
					'code'   => 'newsletter',
					'type'   => 'radio',
					'default_value'  => 1,
					'section' => $this->language->get('text_newsletter'),
					'options' => array(
						'1' => $this->language->get('text_yes'),
						'0' => $this->language->get('text_no'),
					),					
				),
			));
		}
		
		if ($form == 'E') {
			$profile_fields = array(
				'vendor_logo' => array(
					'tab' => 'public',
					'code' => 'vendor_logo',
					'type' => 'image',
					'required' => false,
					'sort_order' => 10,
				),
			);
			
			$fields = array_merge($fields, $profile_fields);
		}
		
		return $fields;
	}
	
	public function getVendorDescription($vendor_id = null, $language_id = null) {
	
		if (empty($vendor_id)) {
			$vendor_id = $this->customer->getId();
		}
		
		$data = parent::getVendorDescription($vendor_id);
		if (empty($language_id)) {
			return $data;
		}
		
		$language_id = $this->config->get('config_language_id');
		if (empty($data[$language_id])) {
			$data = array(
				'name'        => '',
				'description' => '',
			);
		} else {
			$data = $data[$language_id];
		}
		
		return $data;
	}	
	
	
	public function isAwaitingApproval($customer_id = 0) {
		
		$customer_id = $this->customer->getId();
		
		return parent::isAwaitingApproval($customer_id);
	}	
}

class_alias(__NAMESPACE__ . '\ModelVendor', '\ModelExtensionKaExtensionsKaMultivendorVendor');