<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 151 $)
*/

namespace extension\ka_extensions\ka_multivendor;

class ModelVendorProduct extends AModelProduct {

	protected $errors;
	
	private $kamodel_vendor_category;
	
	protected function onLoad() {
	
		$this->kamodel_vendor_category = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/category');
		
		return true;
	}
	
	public function isVendorProduct($product_id) {

		$product = $this->db->query("SELECT vendor_id FROM " . DB_PREFIX . "product
			WHERE product_id = '" . (int)$product_id . "'
		")->row;
		
		if (empty($product)) {
			return false;
		}
		
		if ($product['vendor_id'] != $this->customer->getId()) {
			return false;
		}
		
		return true;
	}

	
	public function saveProduct($data) {

		if (empty($data)) {
			return false;
		}
	
		$data['vendor_id'] = $this->customer->getId();
		
		$vendor = $this->kamodel_vendor->getVendor();
		
		$is_new = true;
		$product_id = null;

		if (empty($data['product_id'])) {
			
			$products_total = $this->getProductsTotal();
			if ($products_total >= $vendor['product_limit']) {
				$this->lastError = $this->language->get("The product limit reached");
				return false;
			}
			
			$product_id = $this->tbl_product->insert($data);
			
		} else {
			$product_id = $data['product_id'];
			if (!$this->isVendorProduct($product_id)) {
				return false;
			}
			
			$is_new = false;
			$this->tbl_product->update($data);
		}

		// save 'special price'
		//
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_special WHERE
			product_id = '" . ($product_id) . "'
		");
		if (!empty($data['special_price'])) {
			$special = array(
				'price' => $data['special_price']
			);
			$this->tbl_product_special->setValue('product_id', $product_id);
			$this->tbl_product_special->save($special);
		}

		// save 'product attributes'
		//
		$this->tbl_product_attribute->setValue('product_id', $product_id);
		
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_attribute WHERE
			product_id = '" . ($product_id) . "'
		");
		
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();

		foreach ($languages as $l) {
			if (!empty($data['product_attributes'][$l['language_id']])) {
				$attrs = $data['product_attributes'][$l['language_id']];
			} else {
				$attrs = array();
			}
			
			if (empty($attrs)) {
				continue;
			}
			
			foreach ($attrs as $attr_id => $attr) {
				if (empty($attr)) {
					continue;
				}
				$rec = array(
					'language_id'  => (int)$l['language_id'],
					'attribute_id' => $attr_id,
					'text'         => $attr
				);
				$this->tbl_product_attribute->save($rec);
			}
		}
		
		// save the main product category
		//
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_to_category WHERE
			product_id = '" . ($product_id) . "'
		");
		
		$rec = array(
			'product_id'  => $product_id,
			'category_id' => (int)$data['category_id']
		);
		$this->tbl_product_to_category->save($rec);
		
		// save 'product images'
		//
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_image WHERE
			product_id = '$product_id'
		");
		$this->tbl_product_image->setValue('product_id', $product_id);
		
		if (!empty($data['product_images'])) {
			foreach ($data['product_images'] as $image) {
				if (!empty($image['image'])) {
					$this->tbl_product_image->save($image);
				}
			}
		}
		
		// save product description
		//
		$this->tbl_product_description->setValue('product_id', $product_id);
		
		foreach ($languages as $l) {
		
			$rec = array();
		
			if (!empty($data['product_description'][$l['language_id']])) {
				$rec = $data['product_description'][$l['language_id']];
				
				// purify description fields
				foreach($rec as $dk => $dv) {
					$text = html_entity_decode($dv, ENT_QUOTES, 'UTF-8');
					$text = HTMLPurifier::purifyCustomerInput($text);
					$rec[$dk] = $text;
				}
			}
			$rec['language_id'] = (int)$l['language_id'];
			
			$this->generateMetaData($rec);
			
			if ($is_new) {
				$this->createSeoUrl($product_id, $rec);
			}
			
			$this->tbl_product_description->save($rec);
		}
		
		// save product for the specific store
		//
		$rec = array(
			'product_id' => $product_id,
			'store_id'   => (int)$this->config->get('config_store_id')
		);
		$this->tbl_product_to_store->save($rec);
		
		return $product_id;
	}
	
	
	protected function generateMetaData(&$rec) {
	
		if (empty($rec['name'])) {
			return;
		}
		$rec['meta_title'] = $rec['name'];

		if (!empty($rec['description'])) {
			$rec['meta_description'] = utf8_substr($rec['description'], 0, 255);
		} else {
			$rec['meta_description'] = $rec['meta_title'];
		}
		
		if (!empty($rec['tag'])) {
			$rec['meta_keyword'] = $rec['tag'];
		} else {
			$rec['meta_keyword'] = '';
		}
	}
	
	/*
		Create seo data URL for new products
	*/
	protected function createSeoUrl($product_id, $rec) {
	
		$keyword = 'product-' . $product_id;
		
		// when the language does not match the user language, we add its code to the url
		//
		if ($this->config->get('config_language_id') != $rec['language_id']) {
			$this->load->model('localisation/language');
			$lang = $this->model_localisation_language->getLanguage($rec['language_id']);
			
			$lang_code = preg_replace('/^(\w*).*/', "$1", $lang['code']);
			if (empty($lang_code)) {
				$lang_code = $rec['language_id'];
			}
			$keyword .= '-' . $lang_code;
		}
	
		$seo = array(
			'store_id'    => $this->config->get('config_store_id'),
			'language_id' => $rec['language_id'],
			'query'       => 'product_id=' . $product_id,
			'keyword'     => $keyword
		);

		$this->kadb->insert('seo_url', $seo);
	}
	
	
	public function deleteProduct($product_id) {
	
		// check vendor access
		//
		$product = $this->getProduct($product_id);
		if (empty($product)) {
			return;
		}

		// copied from the back-end
		//
		$this->db->query("DELETE FROM " . DB_PREFIX . "product WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_attribute WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_description WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_discount WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_filter WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_image WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_option WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_option_value WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE related_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_reward WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_special WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_to_category WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_to_download WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_to_layout WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_to_store WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "product_recurring WHERE product_id = " . (int)$product_id);
		$this->db->query("DELETE FROM " . DB_PREFIX . "review WHERE product_id = '" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "seo_url WHERE query = 'product_id=" . (int)$product_id . "'");
		$this->db->query("DELETE FROM " . DB_PREFIX . "coupon_product WHERE product_id = '" . (int)$product_id . "'");

		$this->cache->delete('product');
	}

		
	public function getProduct($product_id) {

		$product = $this->getProductInternal($product_id);
		if (empty($product)) {
			return array();
		}
		
		if ($product['vendor_id'] != $this->customer->getId()) {
			return array();
		}
		
		$product['unlimited'] = !$product['subtract'];

		return $product;
	}
	
	
	public function getProductDescription($product_id) {
		$product = array();
		
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "product_description WHERE product_id = '" . (int)$product_id . "'");
		
		foreach ($query->rows as $result) {
			$product[$result['language_id']] = $result;
		}
		
		return $product;
	}

	
	protected function getRecords($data = array()) {

		$language_id = (int)$this->config->get('config_language_id');
	
		$where = array();
		if (!empty($data['where'])) {
			$where[] = $data['where'];
		}

		$where[] = "vendor_id = '" . (int)$this->customer->getId() . "'";

		if (isset($data['filter_text'])) {
			$or = array();
			$or[] = "model LIKE '%" . $this->db->escape($data['filter_text']) . "%'";
			$or[] = "pd.name LIKE '%" . $this->db->escape($data['filter_text']) . "%'";
			$or[] = "pd.description LIKE '%" . $this->db->escape($data['filter_text']) . "%'";
			$where[] = '(' . implode(" OR ", $or) . ')';
		}
			
		if (isset($data['filter_quantity'])) {
		
			$qtys = explode('|', $data['filter_quantity']);

			if (!empty($qtys[0])) {
				$where[] = "quantity >= '" . (int)$this->db->escape($qtys[0]) . "'";
			}

			if (!empty($qtys[1])) {
				$where[] = "quantity <= '" . (int)$this->db->escape($qtys[1]) . "'";
			}
		}

		if (isset($data['filter_image'])) {
			$where[] = "image = '" . $this->db->escape($data['filter_image']) . "'";
		}
			
		if (isset($data['filter_manufacturer_id'])) {
			$where[] = "manufacturer_id = '" . $this->db->escape($data['filter_manufacturer_id']) . "'";
		}
			
		if (isset($data['filter_price'])) {
		
			$qtys = explode('|', $data['filter_price']);

			if (!empty($qtys[0])) {
				$where[] = "price >= '" . (int)$this->db->escape($qtys[0]) . "'";
			}

			if (!empty($qtys[1])) {
				$where[] = "price <= '" . (int)$this->db->escape($qtys[1]) . "'";
			}
		}
			
		if (isset($data['filter_status'])) {
			$where[] = "status = '" . $this->db->escape($data['filter_status']) . "'";
		}
		
		if (empty($data['fields'])) {
			$fields = 'p.*';
		} else {
			$fields = $data['fields'];
		}

		$fields .= ", (SELECT price FROM " . DB_PREFIX . "product_special ps WHERE 
			ps.product_id = p.product_id 
			AND ((ps.date_start = '0000-00-00' OR ps.date_start < NOW()) 
			AND (ps.date_end = '0000-00-00' OR ps.date_end > NOW())) 
			ORDER BY ps.priority ASC, ps.price ASC LIMIT 1) 
			AS special
		";
		
		$sql = "SELECT $fields FROM " . DB_PREFIX . "product p";
		
		$sql .= " LEFT JOIN " . DB_PREFIX . "product_description pd 
				ON (p .product_id = pd.product_id 
					AND language_id = $language_id
				)";

		if (!empty($where)) {
			$sql .= " WHERE " . implode(" AND ", $where);
		}
		
		$sort_data = array(
			'p.product_id',
			'pd.name',
			'p.model',
			'p.quantity',
			'p.price',
			'p.status',
		);
		if (!empty($data['sort']) && in_array($data['sort'], $sort_data)) {
		
			$sql .= " ORDER BY " . $data['sort'];
			
			if (!empty($data['order'])) {
				$sql .= ' ' . $data['order'];
			}
		}
		
		if (isset($data['start']) || isset($data['limit'])) {
			if ($data['start'] < 0) {
				$data['start'] = 0;
			}				

			if ($data['limit'] < 1) {
				$data['limit'] = 20;
			}
		
			$sql .= " LIMIT " . (int)$data['start'] . "," . (int)$data['limit'];

		}

		$query = $this->db->query($sql);
		
		return $query;
	}
	
	
	public function getProducts($data = array()) {
	
		$products = array();
	
		$qry = $this->getRecords($data);

		if (!empty($qry->rows)) {
			$products = $qry->rows;
		}
		
		return $products;
	}
	
	
	public function getProductsTotal($data = array()) {
	
      	$data['fields'] = 'COUNT(*) AS total';

		if (isset($data['limit'])) {
			unset($data['limit']);
		}
		if (isset($data['start'])) {
			unset($data['start']);
		}

		$qry = $this->getRecords($data);
		
		if (empty($qry->row)) {
			return 0;
		}

		return $qry->row['total'];
	}
	
	public function getProductFields() {
	
		
	
		$fields = array(
			'product_id'    => array(
				'code'     => 'product_id',
			),		
			// data section 
			'model'    => array(
				'code'     => 'model',
				'required' => true,
				'section' => 'data',
			),
			'mpn'    => array(
				'code'     => 'mpn',
				'section' => 'data',
			),
			'price' => array(
				'code'     => 'price',
				'required' => true,
				'section'  => 'data',
				'format'   => 'price',
			),
			'special_price' => array(
				'code'     => 'special_price',
				'section' => 'data',
				'format'   => 'price',
			),
			'unlimited' => array(
				'code'     => 'unlimited',
				'section' => 'data',
				'type' => 'select',
				'default_value' => 0,
				'options' => array(
					'0' => $this->language->get('txt_no'),
					'1' => $this->language->get('txt_yes'),
				),
			),
			'quantity'    => array(
				'code'     => 'quantity',
				'type'     => 'number',
				'required' => true,
				'section' => 'data',
			),
			'minimum' => array(
				'code'     => 'minimum',
				'section' => 'data',
			),
			'status' => array(
				'code'     => 'status',
				'type' => 'select',
				'section' => 'data',
				'options' => array(
					'1' => $this->language->get('txt_enabled'),
					'0' => $this->language->get('txt_disabled'),
				),
			),

			// section 'Appearance'
			'category_id' => array(
				'code'    => 'category_id',
				'type'    => 'select',
				'section' => 'appearance',
			),
			
			'manufacturer_id' => array(
				'code'     => 'manufacturer_id',
				'type' => 'select',
				'section' => 'appearance',
			),
			
			// section 'Shipping'
			'shipping' => array(
				'code' => 'shipping',
				'section' => 'shipping',
				'default_value' => 1,
				'type' => 'select',
				'options' => array(
					'1' => $this->language->get('txt_required'),
					'0' => $this->language->get('txt_not_required'),
				),
			),
			'weight' => array(
				'code'     => 'weight',
				'section' => 'weight',
				'format' => 'weight',
			),
			'weight_class_id' => array(
				'code'     => 'weight_class_id',
				'section' => '',
				'type' => 'select',
				'required' => true,
			),
			'length' => array(
				'code'     => 'length',
				'section' => 'length',
				'format' => 'length',
			),
			'width' => array(
				'code'     => 'width',
				'section' => '',
				'format' => 'length',
			),
			'height' => array(
				'code'     => 'height',
				'section' => '',
				'format' => 'length',
			),
			'length_class_id' => array(
				'code'     => 'length_class_id',
				'section' => '',
				'type' => 'select',
				'required' => true,
			),
			'image' => array(
				'code'     => 'image',
				'default_value' => '',
				'required' => true,
				'section'  => 'appearance',
				'type'     => 'image',
			),
		);

		
		$currency_sign = $this->currency->getSymbolLeft($this->config->get('config_currency'));
		if (empty($currency_sign)) {
			$currency_sign = $this->currency->getSymbolRight($this->config->get('config_currency'));
		}
		
		$fields['price']['title'] = $this->language->get('txt_title_price') . ' (' . $currency_sign . ')';
		$fields['special_price']['title'] = $this->language->get('txt_title_special_price') . ' (' . $currency_sign . ')';
		
		if (!empty($fields['manufacturer_id'])) {
			$this->load->model('catalog/manufacturer');
			
			$manufacturers = $this->model_catalog_manufacturer->getManufacturers();
			
			$fields['manufacturer_id']['options'][] = $this->language->get('- not specified -');
			
			if (!empty($manufacturers)) {
				foreach ($manufacturers as $m) {
					$fields['manufacturer_id']['options'][$m['manufacturer_id']] = $m['name'];
				}
			}
		}

		
		if (!empty($fields['category_id'])) {
			
			$categories = $this->kamodel_vendor_category->getCategories();
			
			$fields['category_id']['options'][] = $this->language->get('- not specified -');
			
			if (!empty($categories)) {
				foreach ($categories as $cat) {
					$fields['category_id']['options'][$cat['category_id']] = $cat['name'];
				}
			}
		}
		
		if (!empty($fields['weight_class_id'])) {
			$weight_classes = $this->getWeightClasses();
			if (!empty($weight_classes)) {
				foreach ($weight_classes as $wc) {
					$fields['weight_class_id']['options'][$wc['weight_class_id']] = $wc['title'];
				}
			}
		}

		if (!empty($fields['length_class_id'])) {
			$length_classes = $this->getLengthClasses();
			if (!empty($length_classes)) {
				foreach ($length_classes as $wc) {
					$fields['length_class_id']['options'][$wc['length_class_id']] = $wc['title'];
				}
			}
		}
		
		// default model code
		
		$fields['model']['default_value'] = $this->getUniqueModel();
		
		return $fields;
	}

	
	public function getUniqueModel() {
		$max = $this->db->query("SELECT MAX(product_id) AS mx FROM " . DB_PREFIX . "product")->row;
		$model = 'M' . $this->customer->getId() . $max['mx'];
		
		return $model;
	}
	
	public function getProductImageFields() {

		$fields = array(
			'product_id' => array(
				'code'    => 'product_id',
				'type'    => 'text',
			),
			'image' => array(
				'code'    => 'image',
				'default_value' => '',
				'type'    => 'image',
			),
			'sort_order' => array(
				'code'    => 'sort_order',
				'type'    => 'text',
			),
		);	
		
		return $fields;
	}
	

	public function getProductImages($product_id) {
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "product_image WHERE product_id = '" . (int)$product_id . "' ORDER BY sort_order ASC");

		return $query->rows;
	}

	
	public function getProductDescriptionFields() {
	
		$fields = array( 
			'name' => array(
				'code'     => 'name',
				'required' => true,
				'section' => 'description',
			),
			'description' => array(
				'code'     => 'description',
				'type' => 'htmlarea',
				'section' => 'description',
			),
			'tag' => array(
				'code'     => 'tag',
				'section' => 'description',
			),
		);
		
		return $fields;
	}
	
	
	public function getWeightClasses() {
	
		$sql = "SELECT * FROM " . DB_PREFIX . "weight_class wc 
			LEFT JOIN " . DB_PREFIX . "weight_class_description wcd ON (wc.weight_class_id = wcd.weight_class_id) 
			WHERE wcd.language_id = '" . (int)$this->config->get('config_language_id') . "'
		";

		$sql .= " ORDER BY title ASC";

		$classes = $this->db->query($sql)->rows;

		return $classes;
	}

	
	public function getLengthClasses() {
	
		$sql = "SELECT * FROM " . DB_PREFIX . "length_class lc 
			LEFT JOIN " . DB_PREFIX . "length_class_description lcd ON (lc.length_class_id = lcd.length_class_id) 
			WHERE lcd.language_id = '" . (int)$this->config->get('config_language_id') . "'
		";

		$sql .= " ORDER BY title ASC";

		$classes = $this->db->query($sql)->rows;

		return $classes;
	}
	
	public function getProductAttributes($product_id) {
		
		$this->load->model('localisation/language');
		
		$data = array();
		
		$languages = $this->model_localisation_language->getLanguages();
		foreach ($languages as $l) {
			$data[$l['language_id']] = $this->getProductAttributesForLanguage($product_id, $l['language_id']);
		}
		
		return $data;
	}
	
	public function getProductAttributesForLanguage($product_id, $language_id) {
		$attrs = $this->db->query("SELECT a.*, pa.* FROM " . DB_PREFIX . "product_attribute pa 
			LEFT JOIN " . DB_PREFIX . "attribute a ON (pa.attribute_id = a.attribute_id) 
			LEFT JOIN " . DB_PREFIX . "attribute_description ad ON (a.attribute_id = ad.attribute_id) 
			WHERE pa.product_id = '" . (int)$product_id . "' 
				AND pa.language_id = '" . (int)$language_id . "' 
		")->rows;
		
		
		$ret = array();
		foreach ($attrs as $a) {
			$ret[$a['attribute_id']] = $a['text'];
		}

		return $ret;
	}
	

	public function copyProduct($product_id) {
	
		$data = $this->getProduct($product_id);
		
		if (empty($data)) {
			return null;
		}
		
		unset($data['product_id']);
		
		$data['model']   = $this->getUniqueModel();
		$data['sku']     = '';
		$data['upc']     = '';
		$data['viewed']  = '0';
		$data['keyword'] = '';
		$data['status']  = '0';
		
		$data['product_description'] = $this->getProductDescription($product_id);
		foreach ($data['product_description'] as $k => $v) {
			$data['product_description'][$k]['name'] .= ' (' . $this->language->get('copy') . ')';
		}
		
		$data['product_images']      = $this->getProductImages($product_id);
		$data['product_attributes']  = $this->getProductAttributes($product_id);

		$new_product_id = $this->saveProduct($data);
		
		return $new_product_id;
	}
	
	
	public function enableProduct($product_id) {
	
		$data = array(
			'product_id' => $product_id,
			'status' => 1
		);
	
		$this->tbl_product->save($data);
	}
	

	public function disableProduct($product_id) {
	
		$data = array(
			'product_id' => $product_id,
			'status' => 0
		);
	
		$this->tbl_product->save($data);
	}
}

class_alias(__NAMESPACE__ . '\ModelVendorProduct', '\ModelExtensionKaExtensionsKaMultivendorVendorProduct');