<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 154 $)
*/

namespace extension\ka_extensions\ka_multivendor;

use \extension\ka_extensions\QB;

abstract class AModelOrder extends \extension\ka_extensions\Model {

	use TraitGetModel, TraitAddressFormat;

	const ALLOW_COMMENTS_WIHOUT_STATUS_CHANGE = true;

	// notify a vendor when the payment status changes
	//
	const NOTIFY_ON_PAYMENT_STATUS_UPDATE = true;

	protected $kamodel_vendor_transaction;

	/*
		Return if the order has vendor products or not (even there is only one vendor in the order).
	*/
	public function isMultivendorOrder($order_id) {

		$res = $this->db->query("SELECT vendor_id FROM `" . DB_PREFIX . "order_product` WHERE
			order_id = '" . (int)$order_id . "'
			AND	vendor_id 
		")->row;
			
		if (empty($res)) {
			return false;
		}
		
		return true;
	}
	

	protected function getVendorOrderQB($vendor_order_id) {

		$language_id = $this->config->get('config_language_id');
	
		$qb = new QB();
		
		$qb->select("ka_vo.*", "ka_vendor_order", "ka_vo");
		
		$qb->select("(SELECT os.name FROM `" . DB_PREFIX . "order_status` os 
				WHERE os.order_status_id = ka_vo.delivery_status_id 
				AND os.language_id = '" . (int) $language_id . "'
			) AS delivery_status
		");

		$qb->where("vendor_order_id = '" . (int)$vendor_order_id . "'");

		$sql = $qb->getSql();
		
		return $qb;
	}
	
	
	public function getVendorOrder($vendor_order_id) {
	
		$order = $this->getVendorOrderQB($vendor_order_id)->query()->row;

		if (empty($order)) {
			return array();
		}
		
		$model_order = $this->getOrderModel();
		$main_order  = $model_order->getOrder($order['main_order_id']);
	
		$order = array_merge($main_order, $order);

		return $order;
	}

	
	public function getVendorOrderByMainOrder($main_order_id, $vendor_id) {

		$filter = array(
			'vendor_id' => $vendor_id
		);
		
		$orders = $this->getVendorOrdersByMainOrder($main_order_id, $filter);
		if (empty($orders)) {
			return array();
		}
		
		$order = reset($orders);
		
		return $order;
	}


	protected function getVendorOrdersByMainOrderQB($data) {
	
		$qb = new QB();
		
		$qb->select("ka_vo.*", "ka_vendor_order", "ka_vo");
		
		$qb->select("(SELECT os.name FROM " . DB_PREFIX . "order_status os 
				WHERE os.order_status_id = ka_vo.delivery_status_id 
				AND os.language_id = '" . (int)$this->config->get('config_language_id') . "'
			) AS delivery_status"
		);
		
		$qb->where("main_order_id", $data['main_order_id']);
		
		if (isset($data['vendor_id'])) {
			$qb->where("ka_vo.vendor_id", $data['vendor_id']);
		}
		
		return $qb;
	}
	
	
	public function getVendorOrdersByMainOrder($main_order_id, $filter = array()) {

		$data = $filter;
		$data['main_order_id'] = $main_order_id;

		$orders = $this->getVendorOrdersByMainOrderQB($data)->query()->rows;
		if (empty($orders)) {
			return array();
		}
		
		$_orders = array();
		foreach ($orders as $o) {
			$_orders[$o['vendor_id']] = $o;
		}
		$orders = $_orders;
		
		return $orders;
	}
	

	protected function getVendorOrderProductsQB($data) {
	
		$qb = new QB();

		$qb->select("op.*", "order_product", "op");
		$qb->innerJoin("ka_vendor_order", "ka_vo", "op.vendor_id = ka_vo.vendor_id AND op.order_id = ka_vo.main_order_id");
		$qb->where("ka_vo.vendor_order_id", $data['vendor_order_id']);
		
		return $qb;
	}
	
	
	public function getVendorOrderProducts($vendor_order_id) {

		$data = array(
			'vendor_order_id' => $vendor_order_id
		);
	
		$products = $this->getVendorOrderProductsQB($data)->query()->rows;
	
		return $products;
	}
	
	
	/*
		$order - order data before any changes. If it contains order_status_id = 0 then the order was recently placed.
	*/
	public function onMainOrderStatusUpdate($order, $new_order_status_id) {

		// check if the new status is applicable to vendors
		//
		$this->load->model('localisation/order_status');
		$order_status = $this->model_localisation_order_status->getOrderStatus($new_order_status_id);
		
		$vendor_statuses = $this->config->get('ka_multivendor_vendor_statuses');
		if (empty($vendor_statuses)) {
			return;
		}

		// try to find vendor orders
		//
		$kamodel_order = $this->load->kamodel('extension/ka_extensions/ka_multivendor/order');
		$vendor_orders = $kamodel_order->getVendorOrdersByMainOrder($order['order_id']);
		
		if (empty($vendor_orders)) {
			return;
		}
		
		// try to apply the new status to vendor orders
		//
		foreach ($vendor_orders as $vo) {

			$is_vendor_notified = false;
		
			// send a 'New order' email to vendors
			//
			if (empty($order['order_status_id'])) {
				$this->kamodel_order_email->emailVendorNewOrder($order, $vo['vendor_order_id']);
				$is_vendor_notified = true;
			}
		
			// check if we need to update vendor statuses accordingly
			//
			if (in_array($new_order_status_id, $vendor_statuses)) {
				$this->changeVendorStatus($vo['vendor_order_id'], $new_order_status_id, '', false, !$is_vendor_notified);
				if (!$is_vendor_notified) {
					$is_vendor_notified = true;
				}
			}
			
			// send an update to a vendor when the payment status changed and there was no email before
			//
			if (static::NOTIFY_ON_PAYMENT_STATUS_UPDATE) {
				if (!$is_vendor_notified) {
					$this->kamodel_order_email->emailVendorOrderStatusUpdate($vo['vendor_order_id'], '', false, true);
				}
			}
		}
	}
	
	/*
		
	*/
	public function changeVendorStatus($vendor_order_id, $new_order_status_id, $comment = '', $notify_cusotmer = false, $notify_vendor = false) {
	
		$vendor_order = $this->getVendorOrder($vendor_order_id);
		if (empty($vendor_order)) {
			throw new \Exception("Empty vendor order");
		}

		if ($vendor_order['delivery_status_id'] == $new_order_status_id) {
			if (!static::ALLOW_COMMENTS_WIHOUT_STATUS_CHANGE) {
				return false;
			}
		}
		
		// update vendor status
		//
		$this->db->query("UPDATE " . DB_PREFIX . "ka_vendor_order SET
			delivery_status_id = '" . (int)$new_order_status_id . "'
			WHERE vendor_order_id = '" . (int)$vendor_order_id . "'
		");
		
		// add history
		//
		$this->db->query("INSERT INTO " . DB_PREFIX . "order_history SET 
			order_id = '" . (int)$vendor_order['order_id'] . "',
			vendor_order_id = '" . (int)$vendor_order['vendor_order_id'] . "',
			order_status_id = '" . (int)$new_order_status_id . "', 			
			notify = '" . (int)$notify_cusotmer . "', 
			comment = '" . $this->db->escape($comment) . "', 
			date_added = NOW()
		");

		$this->kamodel_vendor_transaction = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/transaction');
		
		$vendor = $this->kamodel_vendor->getVendor($vendor_order['vendor_id']);
		if (!empty($vendor)) {
			// check if we should add transaction to the vendor
			//
			if ($new_order_status_id == $this->config->get('ka_multivendor_confirm_status')) {
				$this->kamodel_vendor_transaction->addTransactionAmount($vendor_order);

			} elseif ($vendor_order['delivery_status_id'] == $this->config->get('ka_multivendor_confirm_status')) {
				$this->kamodel_vendor_transaction->deductTransactionAmount($vendor_order);
			}
		}

		// send an email to the customer about changes to delivery status
		if ($notify_cusotmer) {
			$this->kamodel_order_email->sendDeliveryNote($vendor_order_id, $comment);
		}
		
		
		if ($notify_vendor) {
			$this->kamodel_order_email->emailVendorOrderStatusUpdate($vendor_order_id, $comment);
		}
		
		return true;
	}
	
	
	protected function getOrdersQB($data = array()) {
 
		$qb = new QB();
	
		if (isset($data['limit']) && isset($data['start'])) {
			$qb->limit($data['start'], $data['limit']);
		}		
		
		if (!empty($data['filter_customer'])) {
			$qb->where("CONCAT(o.firstname, ' ', o.lastname) LIKE '%" . $this->db->escape($data['filter_customer']). "%'");
		}
		
		if (!empty($data['filter_placed_from'])) {
			$qb->where("o.date_added >= '" . $this->db->escape($data['filter_placed_from']) . " 00:00:00'");
		}

		if (!empty($data['filter_placed_to'])) {
			$qb->where("o.date_added <= '" . $this->db->escape($data['filter_placed_to']) . " 23:59:59'");
		}
		
		if (empty($data['fields'])) {
			$qb->select('o.*, ka_vo.*');
		} else {
			$qb->select($data['fields']);
		}
		
		$qb->from("ka_vendor_order", 'ka_vo');
		
		$qb->innerJoin("order", "o", "ka_vo.main_order_id = o.order_id");

		$sort_data = array(
			'order_id'        => 'o.order_id',
			'order_status_id' => 'o.order_status_id',
			'date_added'      => 'o.date_added'
		);
		
		if (!empty($data['sort']) && !empty($sort_data[$data['sort']])) {
		
			$order = 'asc';
			if (!empty($data['order'])) {
				if ($data['order'] == 'asc') {
					$data['order'] == 'asc';
				} else {
					$data['order'] == 'asc';
				}
			}
		
			$qb->orderBy($sort_data[$data['sort']] . ' ' . $data['order']);
		}

		return $qb;
	}


	public function getOrders($data = array()) {
	
		$orders = array();
	
		$qry = $this->getOrdersQB($data)->query();

		if (!empty($qry->rows)) {
			$orders = $qry->rows;
		}
		
		return $orders;
	}

	
	public function getOrdersTotal($data = array()) {
	
      	$data['fields'] = 'COUNT(*) AS total';
      	
   		unset($data['sort']);
		unset($data['limit'], $data['start']);

		$qry = $this->getOrdersQB($data)->query();
		
		if (empty($qry->row)) {
			return 0;
		}

		return $qry->row['total'];
	}
	
	
	public function getOrderTotals($vendor_order_id) {

		$order = $this->getVendorOrder($vendor_order_id);

		$products = $this->getVendorOrderProducts($vendor_order_id);
		$subtotal = 0;
		foreach ($products as $p) {
			$subtotal += $p['quantity'] * $p['price'];
		}
	
		$totals = array();

		$totals[] = array(
			'code' => 'subtotal',
			'title' => $this->language->get('Sub-Total'),
			'value' => $subtotal,
		);
		
		$totals[] = array(
			'code' => 'shipping',
			'title' => $this->language->get('Shipping'),
			'value' => $order['shipping_cost'],
		);
		
		$totals[] = array(
			'code' => 'total',
			'title' => $this->language->get('Total'),
			'value' => $order['total'],
		);
		
		return $totals;
	}	
}