<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 120 $)
*/

namespace extension\ka_extensions\ka_multivendor;

use \extension\ka_extensions\QB;

abstract class AModelOrderEmail extends \extension\ka_extensions\Model {

	use TraitGetModel, TraitAddressFormat;

	protected $kamodel_product;
	
	protected function onLoad() {
		$this->kamodel_product = $this->load->kamodel('extension/ka_extensions/ka_multivendor/vendor/product');
	}
	
	
	public function sendDeliveryNote($vendor_order_id, $comment = '') {
	
		$this->load->language('extension/ka_extensions/ka_multivendor/order');

		$data['title'] = $this->language->get('text_shipping');

		if ($this->request->server['HTTPS']) {
			$data['base'] = HTTPS_SERVER;
		} else {
			$data['base'] = HTTP_SERVER;
		}

		$data['direction'] = $this->language->get('direction');
		$data['lang']      = $this->language->get('code');

		$vendor_order = $this->kamodel_order->getVendorOrder($vendor_order_id);
		if (empty($vendor_order)) {
			return;
		}

		$model_order = $this->getOrderModel();
		
		$main_order_id = $vendor_order['main_order_id'];
		
		$vendor = $this->kamodel_vendor->getVendor($vendor_order['vendor_id']);
		
		// Make sure there is a shipping method
		$store_info   = \KaStore::getStoreInfo($vendor_order['store_id']);
		$store_config = \KaStore::getStoreConfig($vendor_order['store_id']);

		$store_address   = $store_config->get('config_address');
		$store_email     = $store_config->get('config_email');
		$store_telephone = $store_config->get('config_telephone');
			
		if ($vendor_order['invoice_no']) {
			$invoice_no = $vendor_order['invoice_prefix'] . $vendor_order['invoice_no'];
		} else {
			$invoice_no = '';
		}

		$shipping_address = $this->getFormattedAddress($vendor_order, 'shipping', $vendor_order['shipping_address_format']);
			
		$product_data = $this->getFormattedProducts($vendor_order);
		if (empty($product_data)) {
			return;
		}

		$delivery_status = $this->kamodel_order_status->getOrderStatus($vendor_order['delivery_status_id']);

		$data['order'] = array(
			'firstname'        => $vendor_order['firstname'],
			'lastname'         => $vendor_order['lastname'],
			'delivery_status'  => $delivery_status['name'],
			'order_id'	       => $vendor_order_id,
			'invoice_no'       => $invoice_no,
			'date_added'       => date($this->language->get('date_format_short'), strtotime($vendor_order['date_added'])),
			'store_name'       => $vendor_order['store_name'],
			'store_url'        => rtrim($vendor_order['store_url'], '/'),
			'store_address'    => nl2br($store_address),
			'store_email'      => $store_email,
			'store_telephone'  => $store_telephone,
			'email'            => $vendor_order['email'],
			'telephone'        => $vendor_order['telephone'],
			'shipping_address' => $shipping_address,
			'shipping_method'  => $vendor_order['shipping_method'],
			'product'          => $product_data,
			'comment'          => nl2br($comment)
		);

		$mail = new \KaMail($vendor_order['language_id'], $vendor_order['store_id']);
		
		$mail->data = $data;
		
		$mail->sendContent(
			$vendor_order['email'], 
			str_replace('%order_id%', $vendor_order['order_id'], $this->language->get('Order update #%order_id%')),
			'extension/ka_extensions/ka_multivendor/mail/order_shipping_content'
		);
		
	}


	public function getFormattedProducts($vendor_order) {
	
		$this->load->model('tool/upload');
		$model_order = $this->getOrderModel();
	
		$product_data = array();
	
		$products = $this->kamodel_order->getVendorOrderProducts($vendor_order['vendor_order_id']);
		if (empty($products)) {
			return $product_data;
		}

		$language = \KaLanguage::getLanguage($vendor_order['language_id']);
		
		foreach ($products as $product) {
			$option_weight = '';

			$product_info = $this->kamodel_product->getProductInternal($product['product_id']);
			
			if (!empty($product_info)) {
				$option_data = array();

				$options = $model_order->getOrderOptions($vendor_order['main_order_id'], $product['order_product_id']);

				foreach ($options as $option) {
					if ($option['type'] != 'file') {
						$value = $option['value'];
					} else {
						$upload_info = $this->model_tool_upload->getUploadByCode($option['value']);

						if ($upload_info) {
							$value = $upload_info['name'];
						} else {
							$value = '';
						}
					}

					$option_data[] = array(
						'name'  => $option['name'],
						'value' => $value
					);

					$product_option_value_info = $this->kamodel_product->getProductOptionValue($product['product_id'], $option['product_option_value_id']);

					if ($product_option_value_info) {
						if ($product_option_value_info['weight_prefix'] == '+') {
							$option_weight += $product_option_value_info['weight'];
						} elseif ($product_option_value_info['weight_prefix'] == '-') {
							$option_weight -= $product_option_value_info['weight'];
						}
					}
				}

				$product_data[] = array(
					'name'     => $product['name'],
					'model'    => $product['model'],
					'option'   => $option_data,
					'quantity' => $product['quantity'],
					'location' => $product_info['location'],
					'sku'      => $product_info['sku'],
					'upc'      => $product_info['upc'],
					'ean'      => $product_info['ean'],
					'jan'      => $product_info['jan'],
					'isbn'     => $product_info['isbn'],
					'mpn'      => $product_info['mpn'],
					'weight'   => $this->weight->format(
						($product_info['weight'] + (float)$option_weight) * $product['quantity'], 
						$product_info['weight_class_id'], 
						$language->get('decimal_point'), 
						$language->get('thousand_point'))
				);
			}
		}
	
		return $product_data;
	}
	
	
	public function emailVendorNewOrder($vendor_order_id) {

		$data = $this->getOrderDataForEmail($vendor_order_id);
		if (empty($data)) {
			return;
		}

		$vendor       = $data['vendor'];
		$vendor_order = $data['vendor_order'];
		
		$mail = new \KaMail($vendor_order['language_id'], $vendor_order['store_id']);
		
		$mail->data = $data;
		
		$mail->sendContent(
			$vendor['email'], 
			str_replace('%order_id%', $vendor_order['order_id'], $this->language->get('New Order #%order_id%')),
			'extension/ka_extensions/ka_multivendor/mail/vendor/order_new'
		);		
	}
	
	
	public function emailVendorOrderStatusUpdate($vendor_order_id, $comment = '') {

		$data = $this->getOrderDataForEmail($vendor_order_id);
		if (empty($data)) {
			return;
		}

		$data['status_comment'] = $comment;
		
		$vendor       = $data['vendor'];
		$vendor_order = $data['vendor_order'];
		
		$mail = new \KaMail($vendor_order['language_id'], $vendor_order['store_id']);
		
		$mail->data = $data;
		
		$mail->sendContent(
			$vendor['email'], 
			str_replace('%order_id%', $vendor_order['order_id'], $this->language->get('Status updated for Order #%order_id%')),
			'extension/ka_extensions/ka_multivendor/mail/vendor/order_update'
		);		
	}
		

	/*
		This is a basic function for sending an email update to the vendor
	*/	
	protected function getOrderDataForEmail($vendor_order_id) {
	
		$this->load->language('mail/order_add');
		$this->load->language('extension/ka_extensions/ka_multivendor/vendor/order');

		$vendor_order = $this->kamodel_order->getVendorOrder($vendor_order_id);
		if (empty($vendor_order)) {
			return;
		}

		if (empty($vendor_order['vendor_id'])) {
			return;
		}

		$vendor = $this->kamodel_vendor->getVendor($vendor_order['vendor_id']);
		
		if (\KaGlobal::isAdminArea()) {
			$model_order = $this->load->kamodel('sale/order');
		} else {
			$model_order = $this->load->kamodel('checkout/order');
		}
		
		$main_order_id = $vendor_order['main_order_id'];
		
		if ($this->request->server['HTTPS']) {
			$data['base'] = HTTPS_SERVER;
		} else {
			$data['base'] = HTTP_SERVER;
		}

		$data['direction'] = $this->language->get('direction');
		$data['lang']      = $this->language->get('code');

		$store_info   = \KaStore::getStoreInfo($vendor_order['store_id']);
		$store_config = \KaStore::getStoreConfig($vendor_order['store_id']);

		$store_address   = $store_config->get('config_address');
		$store_email     = $store_config->get('config_email');
		$store_telephone = $store_config->get('config_telephone');
			
		if ($vendor_order['invoice_no']) {
			$invoice_no = $vendor_order['invoice_prefix'] . $vendor_order['invoice_no'];
		} else {
			$invoice_no = '';
		}

		$shipping_address = $this->getFormattedAddress($vendor_order, 'shipping', $vendor_order['shipping_address_format']);
			
		$product_data = $this->getFormattedProducts($vendor_order);
		if (empty($product_data)) {
			return;
		}

		$data['order'] = array(			
			'firstname'        => $vendor_order['firstname'],
			'lastname'         => $vendor_order['lastname'],
			'order_id'	       => $vendor_order['main_order_id'],
			'invoice_no'       => $invoice_no,
			'date_added'       => date($this->language->get('date_format_short'), strtotime($vendor_order['date_added'])),
			'store_name'       => $vendor_order['store_name'],
			'store_url'        => rtrim($vendor_order['store_url'], '/'),
			'store_address'    => nl2br($store_address),
			'store_email'      => $store_email,
			'store_telephone'  => $store_telephone,
			'email'            => $vendor_order['email'],
			'telephone'        => $vendor_order['telephone'],
			'shipping_address' => $shipping_address,
			'shipping_method'  => $vendor_order['shipping_method'],
			'products'         => $product_data,
		);

		$data['vendor'] = $vendor;
		$data['vendor_order'] = $vendor_order;
		
		return $data;
	}
}