<?php
/*
	$Project: Multivendor $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 1.0.0.0 $ ($Revision: 136 $)
*/

namespace extension\ka_extensions\ka_multivendor;

use extension\ka_extensions\QB;

abstract class AModelVendor extends \extension\ka_extensions\Model {

	protected function getVendorInternal($vendor_id) {
	
		if (empty($vendor_id)) {
			$no_vendor = array(
				'name'      => $this->language->get('Not Specified'),
				'vendor_id' => $vendor_id,
				'store_commission' => 0,
			);
			return $no_vendor;
		}
	
		$language_id = $this->config->get('config_language_id');
		
		$vendor = $this->db->query($x = "SELECT c.*, ka_v.*, ka_vd.name FROM " . DB_PREFIX .  "ka_vendor ka_v
			INNER JOIN " . DB_PREFIX . "customer c ON c.customer_id = ka_v.customer_id
			LEFT JOIN " . DB_PREFIX . "ka_vendor_description ka_vd ON ka_v.customer_id = ka_vd.customer_id
				AND ka_vd.language_id = '$language_id'
			WHERE c.customer_id = '" . (int)$vendor_id . "'
		")->row;

		if (empty($vendor)) {
			return array();
		}

		$vendor['vendor_custom_field'] = json_decode($vendor['vendor_custom_field'], true);
		
		return $vendor;
	}
	
	
	/*
		Returns:
			NULL - not registered
			0    - disabled
			1    - enabled
	*/
	public function getVendorStatus($customer_id) {

		$customer = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_vendor
			WHERE customer_id = '" . (int)$customer_id . "'
		")->row;

		if (empty($customer)) {
			return null;
		}
		
		return $customer['vendor_status'];
	}
	
	
	public function sortArraysBySortOrder($a, $b) {
		if (empty($a['sort_order'])) {
			$a['sort_order'] = 0;
		}
		if (empty($b['sort_order'])) {
			$b['sort_order'] = 0;
		}
		
		if ($a['sort_order'] == $b['sort_order']) {
			return 1;
		}
		
		return ($a['sort_order'] < $b['sort_order']) ? -1 : 1;
	}	

	
	public function isAwaitingApproval($customer_id) {
		
		$customer = $this->db->query("SELECT * FROM " . DB_PREFIX . "customer_approval
			WHERE customer_id = '" . $customer_id . "'
			AND type = 'vendor'
		")->row;
	
		if (empty($customer)) {
			return false;
		}
		
		return true;
	}
	
	
	protected function getVendorsQB($data) {
	
		$qb = new QB();
		
		$qb->select('*, ka_vd.name, c.customer_id AS vendor_id', 'ka_vendor', 'ka_v');
		$qb->innerJoin('customer', 'c', 'c.customer_id = ka_v.customer_id');
		$qb->innerJoin('ka_vendor_description', 'ka_vd', "c.customer_id = ka_vd.customer_id 
			AND ka_vd.language_id = '" . $this->config->get('config_language_id') . "'
		");
		$qb->groupBy('ka_v.customer_id');
		
		return $qb;
	}
	
	public function getVendors($data = array()) {

		$sql = $this->getVendorsQB($data)->getSql();
	
		$result = $this->db->query($sql)->rows;
	
		return $result;
	}
	

	public function addVendor($data) {

		$customer_id = $data['customer_id'];
		if (empty($customer_id)) {
			return false;
		}
	
		$rec = $data;
		$rec['customer_id']       = $customer_id;
		$rec['product_limit']     = (int)(isset($data['product_limit']) ? $data['product_limit'] : $this->config->get('ka_multivendor_default_products_limit'));
		$rec['store_commission']  = (float)(isset($data['store_commission'])) ? $data['store_commission'] : $this->config->get('ka_multivendor_default_commision');
		$rec['vendor_custom_field'] = !empty($data['vendor_custom_field']) ? $data['vendor_custom_field'] : '';
		
		$this->tbl_ka_vendor->insert($rec);
		$this->recordAction($customer_id, 'vendor_added');		
		
		// update the description
		if (!empty($data['vendor_description'])) {
			$this->updateVendorDescription($customer_id, $data['vendor_description']);		
		}
		
		return $customer_id;
	}
	
	
	protected function updateVendorDescription($customer_id, $description) {
	
		$this->load->model('localisation/language');
		$languages = $this->model_localisation_language->getLanguages();
		
		// save vendor description
		//
		$this->tbl_ka_vendor_description->setValue('customer_id', $customer_id);
		
		foreach ($languages as $l) {
		
			$rec = array();
		
			if (!empty($description[$l['language_id']])) {
				$rec = $description[$l['language_id']];
			}
			$rec['language_id'] = (int)$l['language_id'];
			
			$this->tbl_ka_vendor_description->save($rec);
		}
	}
	
	
	public function editVendor($customer_id, $data) {

		$customer_id = (int)$customer_id;
		if (empty($customer_id)) {
			return false;
		}
		
		// update main vendor information
		//
		$this->tbl_ka_vendor->update($data, "customer_id = '$customer_id'");
		
		// save vendor description
		//
		if (!empty($data['vendor_description'])) {
			foreach ($data['vendor_description'] as $lk => $lv) {
				foreach($lv as $dk => $dv) {
					$text = html_entity_decode($dv, ENT_QUOTES, 'UTF-8');
					$text = HTMLPurifier::purifyCustomerInput($text);
					$data['vendor_description'][$lk][$dk] = $text;
				}
			}
			$this->updateVendorDescription($customer_id, $data['vendor_description']);
		}
		
		return $customer_id;
	}
	
	
	protected function recordAction($customer_id, $action) {
	
		$comment = '';
		if ($action == 'vendor_added') {
			$comment = $this->language->get('Vendor profiled was added by %user%');
		} else if ($action == 'vendor_disabled') {
			$comment = $this->language->get('Vendor profiled was disabled by %user%');
		} else if ($action == 'vendor_enabled') {
			$comment = $this->language->get('Vendor profiled was enabled by %user%');
		} else {
			$comment = 'Unkown action by %user%';
		}
	
		$use = '';
		if (\KaGlobal::isAdminArea()) {
			$user = 'admin (id:' . $this->user->getId() . ')';
		} else {
			$user = 'customer';
			if ($this->customer->getId()) {
				$user .= "(" . $this->customer->getId() . ")";
			}
		}
	
		$rec = array(
			'customer_id' => $customer_id,
			'comment'     => str_replace('%user%', $user, $comment),
		);
		$this->tbl_customer_history->insert($rec);
	}
	
	
	public function getVendorDescription($vendor_id = null) {
		
		$vendor = array();
		
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "ka_vendor_description 
			WHERE customer_id = '" . (int)$vendor_id . "'
		");
		
		foreach ($query->rows as $result) {
			$vendor[$result['language_id']] = $result;
		}
		
		return $vendor;
	}
	
	
	public function getVendorDescriptionFields() {
	
		$fields = array( 
			'name' => array(
				'code'     => 'name',
				'required' => true,
				'section'  => 'description',
			),
			'description' => array(
				'code'    => 'description',
				'type'    => 'htmlarea',
				'section' => 'description',
			),
		);
		
		// add automatic tooltips to the fields
		//
		foreach ($fields as &$f) {
			$key = 'txt_help_' . $f['code'];
			$tooltip = $this->language->get($key);
			if ($tooltip != $key) {
				$f['tooltip'] = $tooltip;
			}
		}
		
		return $fields;
	}
}